import {HttpClient, HttpHeaders} from '@angular/common/http';
import {Injectable} from '@angular/core';
import {map} from 'rxjs/operators';
import {AppComponent} from '../app.component';

@Injectable({
  providedIn: 'root'
})
export class AuthenticationService {

  private static API_AUTH_PATH = AppComponent.API_ENDPOINT + '/auth';
  private AUTHENTICATED_USER_PROP_NAME = 'authenticatedUser';
  private user = {
    username: <String> null,
    authData: <String> null
  }
  public username: string;
  public password: string;

  constructor(private httpClient:HttpClient) {

  }

  authenticationService(username: String, password: String) {
    console.log("AUTH S CALL");
    const authData = this.createBasicAuthToken(username, password);
    const headers = new HttpHeaders({ Authorization: authData });
    return this.httpClient.get(AuthenticationService.API_AUTH_PATH,
      {headers: headers}).pipe(map(() => {
        console.log("auth pipe");
        this.registerSuccessfulLogin(username, authData);
    }));
  }

  createBasicAuthToken(username: String, password: String) {
    return 'Basic ' + window.btoa(username + ":" + password)
  }

  registerSuccessfulLogin(username: String, authData: String) {
    this.user.username = username;
    this.user.authData = authData;
    localStorage.setItem(this.AUTHENTICATED_USER_PROP_NAME, JSON.stringify(this.user));
  }

  logout() {
    localStorage.removeItem(this.AUTHENTICATED_USER_PROP_NAME);
    this.user.username = null;
    this.user.authData = null;
  }

  isUserLoggedIn() {
    let user: string = localStorage.getItem(this.AUTHENTICATED_USER_PROP_NAME);
    console.log(user);
    if (user == null || user == 'undefined') return false;
    let jsonUser: any = JSON.parse(user);
    console.log("u " + jsonUser.username);
    console.log("up " + jsonUser.authData);
    return true;
  }

  getLoggedInUser() {
    let user: string = localStorage.getItem(this.AUTHENTICATED_USER_PROP_NAME);
    if (user == null || user == 'undefined') return null;
    return JSON.parse(user);
  }
}
