import { Component, OnInit } from "@angular/core";
import { ActivatedRoute } from "@angular/router";
import { TestService } from "../../service/test.service";
import { MatSnackBar } from "@angular/material";

@Component({
  selector: "app-test-session",
  templateUrl: "./test-session.component.html",
  styleUrls: ["./test-session.component.css"],
})
export class TestSessionComponent implements OnInit {
  constructor(
    private snackBar: MatSnackBar,
    private testService: TestService,
    private route: ActivatedRoute
  ) {
    console.log("constructor test session");
  }

  public sessionTestLink;

  public candidateTest;
  public questionNum;

  public candidateTestOverview;
  public testInProgress;
  public testFinished;

  private testIsNotAvailable;

  public currentQuestion;
  public currentQuestionNumber;

  public answer;
  private answerIds = [];

  public testEndTime;

  progress: number = 100; // Изначальное значение прогресса
  decrementPerSecond: number; // Количество единиц прогресса, которое нужно отнимать в секунду
  interval: any; // Идентификатор интервала
  startTime: number; // Время начала отсчета в миллисекундах
  elapsedTime: number; // Прошедшее время в миллисекундах

  calculateProgress(elapsedSeconds: number): number {
    const percentPerSecond = 1; // Количество процентов, которые должны убывать в секунду
    const totalSeconds =
      this.candidateTest.duration -
      (elapsedSeconds % this.candidateTest.duration);

    return 100 - percentPerSecond * totalSeconds;
  }

  startTimer() {
    const intervalDuration =
      (this.candidateTest.duration * 1000) / this.progress; // Интервал в миллисекундах

    const interval = setInterval(() => {
      if (this.elapsedTime > 0) {
        // Рассчитайте прошедшее время в секундах
        const elapsedSeconds = this.elapsedTime / 1000;
        // Рассчитайте прогресс на основе прошедшего времени
        this.progress = this.calculateProgress(elapsedSeconds);
      } else {
        this.progress -= 1;
      }

      if (this.progress <= 0) {
        clearInterval(interval); // Останавливаем таймер, когда прогресс достигает нуля
      }
    }, intervalDuration);
  }

  ngOnInit() {
    //TODO if test started load question
    this.route.paramMap.subscribe((params) => {
      this.sessionTestLink = params.get("link");
      if (this.sessionTestLink != null) {
        this.candidateTestOverview = true;
        this.loadCandidateTest(this.sessionTestLink);
      }
    });

    console.log("INIt test session");
  }

  loadCandidateTest(link) {
    this.testService.getCandidateTest(link).subscribe(
      (response) => {
        if (response["status"] != "success") {
          //TODO fix error handling
          console.log("Error loading candidate test");
          return;
        }
        this.candidateTest = response["data"];

        switch (this.candidateTest.status) {
          case "NEW":
            this.candidateTestOverview = true;
            break;
          case "IN_PROGRESS":
            const startTime = new Date(
              this.candidateTest.testStarted
            ).getTime();

            const now = new Date().getTime();
            const duration = this.candidateTest.duration * 1000;

            this.elapsedTime = now - startTime;
            this.progress = (this.elapsedTime / duration) * 100;
            this.startTimer();

            this.candidateTestOverview = false;
            this.testInProgress = true;
            this.testEndTime = startTime + duration;
            this.getNextQuestion();

            break;
          case "FINISHED":
            this.candidateTestOverview = false;
            this.testInProgress = false;
            this.testFinished = true;
            break;
        }
      },
      (error) => {
        console.log("ERROR GET CANDIDATE TEST WITH LINK: " + link);
        this.testIsNotAvailable = true;
      }
    );
  }

  startCandidateTest = () => {
    this.candidateTestOverview = false;

    this.testService.startCandidateTest(this.sessionTestLink).subscribe(
      (response) => {
        if (response["status"] != "success") {
          //TODO fix error handling
          console.log("Error loading candidate test");
          return;
        }
        this.testInProgress = true;
        this.snackBar.open("Тест начат", "Ок", {
          duration: 3000,
        });
        this.decrementPerSecond = this.progress / this.candidateTest.duration;
        this.startTimer();

        this.testEndTime =
          new Date().getTime() + this.candidateTest.duration * 1000;
        this.getNextQuestion();
      },
      (error) => {
        console.log(
          "ERROR STARTING CANDIDATE TEST WITH LINK: " + this.sessionTestLink
        );
        this.testIsNotAvailable = true;
      }
    );
  };

  getNextQuestion = () => {
    this.testService.getCandidateTestQuestions(this.sessionTestLink).subscribe(
      (response) => {
        if (response["status"] != "success") {
          //TODO fix error handling
          console.log("Error loading candidate test questions");
          return;
        }
        let count = 0;
        for (var i = 0; i < response["data"].length; ++i) {
          if (response["data"][i].answered === true) {
            count++;
          }
        }
        this.currentQuestionNumber = count + 1;
        let question = response["data"].find(function (q) {
          return q.answered === false;
        });

        if (question == null) {
          this.testService.finishCandidateTest(this.sessionTestLink).subscribe(
            (response) => {
              if (response["status"] != "success") {
                //TODO fix error handling
                console.log("Error finishing test");
                return;
              }
              this.testFinished = true;
              this.testInProgress = false;
            },
            (error) => {
              console.log(
                "ERROR FINISHING CANDIDATE TEST WITH LINK: " +
                  this.sessionTestLink
              );
            }
          );

          this.testFinished = true;
          this.testInProgress = false;
          return;
        }
        this.currentQuestion = question;
      },
      (error) => {
        console.log(
          "ERROR GETTING CANDIDATE TEST QUESTIONS WITH LINK: " +
            this.sessionTestLink
        );
      }
    );
  };

  submitQuestionAnswer = () => {
    switch (this.currentQuestion.type) {
      case "Single":
        if (this.answer == null) {
          this.snackBar.open("Выберите ответ!", "Ок", {
            duration: 6000,
          });
          return;
        }
        this.answerIds.push(this.answer.id);
        break;
      case "Multiple":
        //already filled by checkbox onchange
        if (this.answerIds == null || this.answerIds.length == 0) {
          this.snackBar.open("Выберите ответ!", "Ок", {
            duration: 6000,
          });
          return;
        }
        break;
    }

    let postAnswer = {
      questionId: this.currentQuestion.id,
      answerIds: this.answerIds,
    };

    this.testService
      .postCandidateTestAnswer(this.sessionTestLink, postAnswer)
      .subscribe(
        (response) => {
          if (response["status"] != "success") {
            //TODO fix error handling
            console.log("Error loading candidate test questions");
            return;
          }
          this.currentQuestion = null;
          this.answer = null;
          this.answerIds = [];
          this.getNextQuestion();
        },
        (error) => {
          console.log(
            "ERROR POSTING CANDIDATE TEST ANSWER WITH LINK: " +
              this.sessionTestLink
          );
        }
      );
  };

  onChangeAnswerCheckBox = (event) => {
    const answerId = event.source.value.id;
    if (event.checked) {
      this.answerIds.push(answerId);
    } else {
      this.answerIds.splice(this.answerIds.indexOf(answerId), 1);
    }
  };

  onTestTimeExp = (event) => {
    if (event.left === 0) {
      this.testFinished = true;
      this.testInProgress = false;
    } else if (event.left === 300000) {
      this.snackBar.open("Внимание! Осталось 5 минут", "Ок", {
        duration: 6000,
      });
    } else if (event.left === 60000) {
      this.snackBar.open("Внимание! Осталась 1 минута", "Ок", {
        duration: 6000,
      });
    }
  };
}
