import {Component, OnInit, ViewChildren} from '@angular/core';
import {TestService} from "../../../service/test.service";
import {ActivatedRoute, Router} from "@angular/router";
import {FormBuilder, FormGroup} from "@angular/forms";
import {MatSnackBar} from "@angular/material";

@Component({
  selector: 'app-test-edit',
  templateUrl: './test-edit.component.html',
  styleUrls: ['./test-edit.component.css']
})
export class TestEditComponent implements OnInit {

  constructor(private snackBar: MatSnackBar, private testService: TestService, private route: ActivatedRoute,
              private router: Router, private fb: FormBuilder) { }

  public testForm: FormGroup;
  private testId;
  private test;
  public testQuestions = [];
  public topicList: [];

  @ViewChildren('questionList')
  private questionListView;

  ngOnInit() {

    this.createTestForm();

    this.route.paramMap.subscribe(params => {
      this.testId = params.get("id");

      this.fillFormTestData();

    });

  }

  createTestForm() {
    this.testForm = this.fb.group({
      id: [''],
      name: [''],
      description: [''],
      duration: [''],
      numberOfQuestions: [''],
      topics: ['']
    })
  }

  setTopicsInput(questions) {
    var topicIds = [];
    questions.forEach(q => topicIds.push(q.topicId));
    this.testForm.controls.topics.patchValue(topicIds);
  }

  fillFormTestData() {
    this.testService.getTopics().subscribe(response => {

      if (response['status'] === 'success') {
        this.topicList = response['data'];
      } else {
        console.log('error getting topics ' + response['error']);
      }
    }, error => { console.log("ERROR GETING TOPICS"); });

    if (this.testId != null) {
      this.testService.getTest(this.testId).subscribe(response => {

        const data = response['data'];
        this.test = data['test'];
        this.testForm.patchValue(this.test);
        this.testQuestions = data['questions'];
        this.testForm.controls.numberOfQuestions.patchValue(this.testQuestions.length);
        this.testForm.controls.numberOfQuestions.disable();
        this.setTopicsInput(this.testQuestions);
        this.testForm.controls.topics.disable();

      }, error => console.log("ERROR GET TEST WITH ID: " + this.testId));
    }

  }

  submitTestForm() {
    var test = {
      test: {},
      topicIds: {},
      questionNumber: 0
    };

    test.test = this.testForm.value;
    test.topicIds = this.testForm.controls.topics.value;
    test.questionNumber = this.testForm.controls.numberOfQuestions.value;

    this.saveTest(test);
  }

  saveTest(test) {

    if (null == test.test["id"] || "" == test.test["id"]) {
      delete test.test["id"];
      delete test['questions'];

      this.testService.createTest(JSON.stringify(test)).subscribe(response => {

        if (response['status'] === 'success') {
          this.snackBar.open('Тест успешно сохранен', null, {
            duration: 4000
          });
          this.router.navigate(['/test/list']);
        } else {
          this.snackBar.open('Не удалось сохранить! Ошибка: ' + response['data'].error.code + ' ' + response['data'].error.message,
            'Попробовать позже', {
            duration: 15000
          });
        }

      }, error => {
        console.log("ERROR saving test create: " + this.testId);
        this.snackBar.open('Не удалось сохранить! Ошибка: ' + error, 'Попробовать позже', {
            duration: 15000
          });
      });
    } else {
      this.testService.updateTest(JSON.stringify(test.test)).subscribe(response => {

        if (response['status'] === 'success') {
          this.snackBar.open('Тест успешно сохранен', 'Ок', {
            duration: 3000
          });
        } else {
          this.snackBar.open('Не удалось сохранить! Ошибка: ' + response['data'].error.code + ' ' + response['data'].error.message,
            'Попробовать позже', {
              duration: 15000
            });
        }

      }, error => {
        console.log("ERROR saving test upd: " + this.testId);
        this.snackBar.open('Не удалось сохранить! Ошибка: ' + error,'Попробовать позже', {
            duration: 15000
          });
      });
    }

  }

}
