import {Component, Inject, OnInit} from '@angular/core';
import {TestService} from "../../../service/test.service";
import {Router} from "@angular/router";
import {MatSnackBar} from '@angular/material/snack-bar';
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from '@angular/material';

@Component({
  selector: 'app-test',
  templateUrl: './test-list.component.html',
  styleUrls: ['./test-list.component.css']
})
export class TestListComponent implements OnInit {

  public displayedColumns: string[] = ['id', 'name', 'duration', 'control', 'genTestSession'];
  public tests = [];

  constructor(private snackBar: MatSnackBar, private testService: TestService, private router: Router,
    private dialog: MatDialog) { }

  ngOnInit() {
    this.testService.getTests().subscribe( (response) => {

      this.tests = response['data'];

    }, error => console.log(error.message));
  }

  redirectEditTest(testId) {
    this.router.navigate(['/test/edit/' + testId]);
  }

  createCandidateTest(testId) {
    const dialogRef = this.dialog.open(CandidateTestNoteDialog, {
      width: '250px',
      data: {note: ''}
    });

    dialogRef.afterClosed().subscribe(result => {
      this.testService.createCandidateTest(testId, result).subscribe(response => {

        if (response['status'] === 'success') {
          this.snackBar.open('Сессия создана для кандидата. Перейдите на страницу результаты', 'Ок', {
            duration: 5000
          });
        }

      }, error => console.log("ERROR CREATE CANDIDATE TEST WITH ID: " + testId));
    });
  }

}

export interface CandidateTestNoteDialogModel {
  note: string;
}

@Component({
  selector: 'candidate-test-note-dialog',
  templateUrl: 'candidate-test-note-dialog.html',
})
export class CandidateTestNoteDialog{

  constructor(
    public dialogRef: MatDialogRef<CandidateTestNoteDialog>,
    @Inject(MAT_DIALOG_DATA) public data: CandidateTestNoteDialogModel) {}

  onNoClick(): void {
    this.dialogRef.close();
  }

}
