import {Injectable} from '@angular/core';
import {HttpClient} from "@angular/common/http";
import {AppComponent} from "../app.component";


@Injectable({
  providedIn: 'root'
})
export class TestService {

  private static API_TESTS_PATH = AppComponent.API_ENDPOINT + '/tests';
  private static API_TOPICS_PATH = AppComponent.API_ENDPOINT + '/topics';
  private static API_CANDIDATE_TESTS_PATH = AppComponent.API_ENDPOINT + '/public/';

  constructor(private httpClient: HttpClient) { }

  getTopics() {
    return this.httpClient.get(TestService.API_TOPICS_PATH);
  }

  getTests() {
    return this.httpClient.get(TestService.API_TESTS_PATH);
  }

  updateTest(test) {
    return this.httpClient.put(TestService.API_TESTS_PATH, test);
  }

  getTest(id) {
    return this.httpClient.get(TestService.API_TESTS_PATH + "/" + id);
  }

  createTest(testAndTopics) {
    return this.httpClient.post(TestService.API_TESTS_PATH, testAndTopics);
  }

  createCandidateTest(testId, note) {
    let candidateTestRequest = {
      "testId": testId,
      "note": note
    };
    return this.httpClient.post(TestService.API_CANDIDATE_TESTS_PATH, candidateTestRequest);
  }

  getCandidateTests() {
    return this.httpClient.get(TestService.API_CANDIDATE_TESTS_PATH);
  }

  getCandidateTest(link) {
    return this.httpClient.get(TestService.API_CANDIDATE_TESTS_PATH + link);
  }

  startCandidateTest(link) {
    return this.httpClient.put(TestService.API_CANDIDATE_TESTS_PATH + link + "/start", null);
  }
  finishCandidateTest(link) {
    return this.httpClient.put(TestService.API_CANDIDATE_TESTS_PATH + link + "/finish", null);
  }


  getCandidateTestQuestions(link) {
    const size = 5000;
    return this.httpClient.get(TestService.API_CANDIDATE_TESTS_PATH + link + "/questions" + "?size=" + size);
  }

  postCandidateTestAnswer(link, candidateAnswer) {
    return this.httpClient.put(TestService.API_CANDIDATE_TESTS_PATH + link + "/questions", candidateAnswer);
  }

}
