DROP TABLE orders IF EXISTS;
DROP SEQUENCE order_seq IF EXISTS;
DROP VIEW orders_view IF EXISTS;

CREATE SEQUENCE order_seq
    AS INTEGER
    START WITH 100000;

CREATE TABLE orders
(
    id                INTEGER GENERATED BY DEFAULT AS SEQUENCE order_seq PRIMARY KEY,
    external_order_id varchar(64)    NOT NULL,
    payment_method    varchar(32)    NOT NULL,
    amount            numeric(10, 2) NOT NULL,
    test_quantity     INTEGER        NOT NULL,
    subscription_days INTEGER        NOT NULL,
    order_status      varchar(32)    NOT NULL,
    rate_id           BIGINT         NOT NULL,
    user_id           BIGINT         NOT NULL,
    order_date        timestamp without time zone default now(),
    payment_date      timestamp without time zone,
    documents_requested_date    timestamp without time zone,
    documents_responded_date    timestamp without time zone,
    documents_post_name         character varying(255),
    documents_post_address      character varying(255),
    sys_operation     character varying(255),
    sys_ds            timestamp without time zone,
    sys_name          character varying(255),
    sys_login         character varying(255)
);

CREATE VIEW public.orders_view AS
SELECT
    o.id,
    o.external_order_id,
    o.payment_method,
    o.amount,
    o.test_quantity,
    o.order_status,
    o.rate_id,
    o.user_id,
    u.email,
    null as legal_name_comp,
    o.order_date,
    o.payment_date,
    (o.documents_requested_date is not null) as documents_requested,
    (o.documents_responded_date is not null) as documents_responded,
    o.documents_requested_date,
    o.documents_responded_date,
    o.documents_post_name,
    o.documents_post_address,
    o.sys_operation,
    o.sys_ds,
    o.sys_name,
    o.sys_login
FROM orders o
         JOIN users u ON o.user_id = u.id;

