package com.example.new_test_lab.aspect;

import com.example.new_test_lab.controller.response.FailResponse;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import lombok.extern.log4j.Log4j2;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;


import java.util.stream.Collectors;

@Log4j2
@RestControllerAdvice
public class GlobalExceptionHandler {

    @ExceptionHandler(Exception.class)
    public Object handleException(Exception e) {
        log.error(e.toString(), e);
        if (e instanceof ServerException)
            return new FailResponse((ServerException) e);
        else if (e instanceof MethodArgumentNotValidException) {
            MethodArgumentNotValidException exception = (MethodArgumentNotValidException) e;
            String message = exception.getBindingResult().getAllErrors()
                    .stream().map(ObjectError::getDefaultMessage).distinct().collect(Collectors.joining(", ", "", ""));
            return new FailResponse(Exceptions.BAD_REQUEST.getCode(), message);
        } else
            return new FailResponse(Exceptions.SYSTEM_ERROR);
    }
}
