package com.example.new_test_lab.aspect;

import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.stereotype.Component;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;

@Aspect
@Component
public class UserIdentifierConnectionPreparer {

    @AfterReturning(pointcut = "execution(* *.getConnection(..))", returning = "connection")
    public Connection prepare(Connection connection) throws SQLException {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        String userName = "";
        String login = "";
        if (authentication != null && authentication.isAuthenticated() && !(authentication instanceof AnonymousAuthenticationToken)) {
            User currentUser = (User) authentication.getPrincipal();
            userName = currentUser.getUsername();
            login = currentUser.getUsername();
        }
        CallableStatement cs = connection.prepareCall(
                "{ call context_user(?, ?) }");
        cs.setString(1, userName);
        cs.setString(2, login);
        cs.execute();
        cs.close();
        return connection;
    }
}
