package com.example.new_test_lab.config;

import com.example.new_test_lab.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.BeanIds;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CommonsRequestLoggingFilter;
import org.springframework.web.filter.CorsFilter;

import java.util.Arrays;
import java.util.Collections;


@EnableWebSecurity
@Configuration
public class WebAppSecurity extends WebSecurityConfigurerAdapter {

    private static final String ROLE_USER = "USER";

    private static final String ROLE_ADMIN = "ADMIN";

    private static final String ROLE_EDITOR = "EDITOR";

    private static final String ROLE_SUPERUSER = "SUPERUSER";

    @Autowired
    private UserService userService;

    @Autowired
    public void configureGlobal(AuthenticationManagerBuilder auth, PasswordEncoder encoder)
            throws Exception {
        auth
                .userDetailsService(userService)
                .passwordEncoder(encoder);
    }

    @Bean(BeanIds.AUTHENTICATION_MANAGER)
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    @Override
    protected void configure(HttpSecurity httpSecurity)
            throws Exception {
        httpSecurity
                .addFilterBefore(logFilter(), BasicAuthenticationFilter.class)
                .sessionManagement()
                .sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .csrf().disable()
                .authorizeRequests()
                .antMatchers(HttpMethod.DELETE, "/topics/**", "/sections/**", "/questions/**").hasRole(ROLE_EDITOR)
                .antMatchers(HttpMethod.POST, "/topics/**", "/sections/**", "/questions/**").hasRole(ROLE_EDITOR)
                .antMatchers(HttpMethod.PUT, "/topics/**", "/sections/**", "/questions/**").hasRole(ROLE_EDITOR)
                .antMatchers(HttpMethod.GET, "/topics", "/sections", "/questions", "/topics/**", "/sections/**", "/questions/**").hasAnyRole(ROLE_EDITOR, ROLE_USER, ROLE_ADMIN)
                .antMatchers("/public/**", "/test-result/**", "/users/register", "/users/reset-password", "/orders/status/tinkoff").permitAll()
                .antMatchers("/topics", "/tests/**", "/candidate-tests", "/candidate-tests/**").hasAnyRole(ROLE_USER, ROLE_ADMIN)
                .antMatchers("/params").hasRole(ROLE_SUPERUSER)
                .antMatchers(HttpMethod.GET, "/users/balance").hasAnyRole(ROLE_ADMIN, ROLE_USER)
                .antMatchers(HttpMethod.GET, "/rates").hasAnyRole(ROLE_ADMIN, ROLE_SUPERUSER)
                .antMatchers("/rates/*", "/rates").hasRole(ROLE_SUPERUSER)
                .antMatchers(HttpMethod.PUT, "/orders").hasRole(ROLE_SUPERUSER)
                .antMatchers("/v2/api-docs", "/configuration/ui", "/swagger-resources/**", "/configuration/security", "/swagger-ui.html", "/webjars/**").permitAll()
                .anyRequest().hasAnyRole(ROLE_USER, ROLE_ADMIN, ROLE_SUPERUSER, ROLE_EDITOR)
                .and()
                .cors()
                .and()
                .httpBasic();
    }

    @Bean
    public CommonsRequestLoggingFilter logFilter() {
        CommonsRequestLoggingFilter filter = new CommonsRequestLoggingFilter();
        filter.setIncludeQueryString(true);
        filter.setIncludePayload(true);
        filter.setMaxPayloadLength(10000);
        filter.setIncludeHeaders(false);
        filter.setBeforeMessagePrefix("Before request: [");
        filter.setBeforeMessageSuffix("]");
        filter.setAfterMessagePrefix("After request: [");
        filter.setAfterMessageSuffix("]");
        return filter;
    }
    @Bean
    public CorsFilter corsFilter() {
        final UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        final CorsConfiguration config = new CorsConfiguration();
        config.setAllowCredentials(true);
        // Don't do this in production, use a proper list  of allowed origins
        config.setAllowedOriginPatterns(Collections.singletonList("*"));
        config.setAllowedHeaders(Arrays.asList("*"));
        config.setAllowedMethods(Arrays.asList("GET", "POST", "PUT", "OPTIONS", "DELETE", "PATCH"));
        source.registerCorsConfiguration("/**", config);
        return new CorsFilter(source);
    }
}
