package com.example.new_test_lab.config;

import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.validation.beanvalidation.LocalValidatorFactoryBean;
import org.springframework.web.servlet.LocaleResolver;
import org.springframework.web.servlet.config.annotation.*;
import org.springframework.web.servlet.handler.BeanNameUrlHandlerMapping;
import org.springframework.web.servlet.i18n.CookieLocaleResolver;
import org.springframework.web.servlet.i18n.LocaleChangeInterceptor;
import org.springframework.web.servlet.theme.ThemeChangeInterceptor;

import java.util.Locale;

@Configuration
@EnableWebMvc
@EnableScheduling
@ComponentScan(basePackages = {"com.example.new_test_lab.*"})
public class WebMvcConfig implements WebMvcConfigurer {

    //check https://www.baeldung.com/maven-webjars
    private static final String RESOURCE_PATH_PATTERN_WEBJARS = "/webjars/**";

    private static final String RESOURCE_LOCATION_WEBJARS = "classpath:/META-INF/resources/webjars/";

    private static final String RESOURCE_PATH_PATTERN_SWAGGER = "/swagger-ui.html";

    private static final String RESOURCE_LOCATION_SWAGGER = "classpath:/META-INF/resources/";

    private static final String RESOURCE_PATH_PATTERN_API_DOCS = "/v2/api-docs";

    private static final String RESOURCE_LOCATION_API_DOCS = "classpath:/META-INF/resources/";

    private static final String LOCALE_LANGUAGE_VOCABULARY_PATH = "classpath:/locale/language";

    private static final String LOCALE_ERRORS_PATH = "classpath:/locale/errors";

    private static final String DEFAULT_ENCODING = "UTF-8";

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        registry.addResourceHandler(RESOURCE_PATH_PATTERN_SWAGGER)
                .addResourceLocations(RESOURCE_LOCATION_SWAGGER);
        registry.addResourceHandler(RESOURCE_PATH_PATTERN_API_DOCS)
                .addResourceLocations(RESOURCE_LOCATION_API_DOCS);
        registry.addResourceHandler(RESOURCE_PATH_PATTERN_WEBJARS)
                .addResourceLocations(RESOURCE_LOCATION_WEBJARS);
    }

    @Bean("messageSource")
    public MessageSource messageSource() {
        ReloadableResourceBundleMessageSource messageSource = new ReloadableResourceBundleMessageSource();
        messageSource.setBasenames(LOCALE_LANGUAGE_VOCABULARY_PATH, LOCALE_ERRORS_PATH);
        messageSource.setDefaultEncoding(DEFAULT_ENCODING);
        messageSource.setUseCodeAsDefaultMessage(true);
        return messageSource;
    }

    @Bean
    public LocalValidatorFactoryBean getValidator() {
        LocalValidatorFactoryBean bean = new LocalValidatorFactoryBean();
        bean.setValidationMessageSource(messageSource());
        return bean;
    }

    @Bean
    public LocaleResolver localeResolver() {
        CookieLocaleResolver resolver = new CookieLocaleResolver();
        resolver.setDefaultLocale(new Locale("ru", "RU"));
        return new CookieLocaleResolver();
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        ThemeChangeInterceptor themeChangeInterceptor = new ThemeChangeInterceptor();
        themeChangeInterceptor.setParamName("theme");
        registry.addInterceptor(themeChangeInterceptor);

        LocaleChangeInterceptor localeChangeInterceptor = new LocaleChangeInterceptor();
        localeChangeInterceptor.setParamName("lang");
        registry.addInterceptor(localeChangeInterceptor);
    }

    @Override
    public void addCorsMappings(CorsRegistry registry) {
        registry.addMapping("/**").allowedMethods("GET", "POST", "PUT", "DELETE", "OPTIONS")
                .allowedHeaders("*")
                .allowedOrigins("*")
                .allowedOriginPatterns("*");
    }

    @Bean
    public BeanNameUrlHandlerMapping beanNameUrlHandlerMapping() {
        return new BeanNameUrlHandlerMapping();
    }
}
