package com.example.new_test_lab.controller;

import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.CompanyInfoDto;
import com.example.new_test_lab.mapper.CompanyInfoMapper;
import com.example.new_test_lab.model.CompanyInfo;
import com.example.new_test_lab.model.User;
import com.example.new_test_lab.service.CompanyInfoService;
import com.example.new_test_lab.service.UserService;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;



import static com.example.new_test_lab.util.SecurityUtil.authUser;


@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/company-info")
public class CompanyInfoController {

    private final CompanyInfoService companyInfoService;

    private final UserService userService;

    private final CompanyInfoMapper companyInfoMapper;

    @Autowired
    public CompanyInfoController(CompanyInfoService companyInfoService, CompanyInfoMapper companyInfoMapper, UserService userService) {
        this.companyInfoService = companyInfoService;
        this.companyInfoMapper = companyInfoMapper;
        this.userService = userService;
    }

    @ApiOperation("Получение информации о компании")
    @GetMapping( produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getCompanyInfo() {
        User user = authUser();
        log.info("#get by id: {}", user.getCompanyInfoId());
        if (user.getCompanyInfoId() == 0) return new SuccessResponse(new CompanyInfo());
        return new SuccessResponse(companyInfoService.findCompanyInfoById(user.getCompanyInfoId()));
    }

    @ApiOperation( "Создание информации о компании")
    @PostMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object save(@Valid @RequestBody CompanyInfoDto companyInfoDto) {
        log.info("#save companyInfo: {}", companyInfoDto);
        CompanyInfo companyInfo = companyInfoMapper.toCompanyInfo(companyInfoDto);
        User user = authUser();
        if (user.getCompanyInfoId() != 0) {
            CompanyInfo companyInfoExist = companyInfoService.findCompanyInfoById(user.getCompanyInfoId());
            companyInfo.setId(companyInfoExist.getId());
        } else companyInfo.setId(0L);
        companyInfo = companyInfoService.save(companyInfo);
        userService.updateCompanyInfoId(user, companyInfo.getId());
        return new SuccessResponse(companyInfo);
    }

}
