package com.example.new_test_lab.controller;


import com.example.new_test_lab.controller.response.ListResponse;
import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.CandidateAnswerDto;
import com.example.new_test_lab.dto.CandidateChoiceDto;
import com.example.new_test_lab.dto.CandidateQuestionDto;
import com.example.new_test_lab.model.CandidateAnswer;
import com.example.new_test_lab.model.CandidateQuestion;
import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.service.CandidateQuestionService;
import com.example.new_test_lab.service.CandidateTestService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.AppUtil.countPages;


@Slf4j
@RestController
@RequestMapping("/public")
public class PublicController {

    private CandidateTestService testService;
    private CandidateQuestionService questionService;

    @Autowired
    public PublicController(CandidateTestService testService, CandidateQuestionService questionService) {
        this.testService = testService;
        this.questionService = questionService;
    }

    @ApiOperation("Получение теста по коду")
    @GetMapping(value = "/{linkCode}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable String linkCode) {
        log.info("#get by linc code: {}", linkCode);
        return new SuccessResponse(testService.getByLinkCode(linkCode));
    }

    @ApiOperation("Начало теста по коду")
    @PutMapping(value = "/{linkCode}/start", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object start(@PathVariable String linkCode) {
        log.info("#start test by link code: {}", linkCode);

        testService.start(linkCode);
        return new SuccessResponse();
    }

    @ApiOperation("Завершение теста по коду")
    @PutMapping(value = "/{linkCode}/finish", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object finish(@PathVariable String linkCode) {
        log.info("#finish test by link code: {}", linkCode);

        testService.finish(linkCode);
        return new SuccessResponse();
    }

    @ApiOperation("Получение вопросов теста по коду")
    @GetMapping(value = "/{linkCode}/questions", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getQuestions(@PathVariable String linkCode,
                               @RequestParam(required = false, defaultValue = "0") int page,
                               @RequestParam(required = false, defaultValue = "10") int size) {

        log.info("#get questions by link code: {}", linkCode);

        CandidateTest candidateTest = testService.getByLinkCode(linkCode);
        List<CandidateQuestion> questions = questionService.getQuestionsByLinkCode(linkCode, page, size);
        Map<Long, List<CandidateAnswer>> answersMap = questionService.getAnswers(questions.stream().mapToLong(CandidateQuestion::getId).toArray()).stream().collect(Collectors.groupingBy(CandidateAnswer::getCandidateQuestionId));

        return new ListResponse(questions.stream().map(q ->
                new CandidateQuestionDto(q, answersMap.get(q.getId()).stream().map(ca -> new CandidateAnswerDto(ca, false)).collect(Collectors.toList()))
        ).collect(Collectors.toList()), countPages(candidateTest.getQuestionNumber(), size), candidateTest.getQuestionNumber());
    }

    @ApiOperation("Отправка ответа на вопрос по коду")
    @PutMapping(value = "/{linkCode}/questions", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object answer(@PathVariable String linkCode, @RequestBody CandidateChoiceDto request) {
        questionService.answer(linkCode, request.getQuestionId(), request.getAnswerIds());
        return new SuccessResponse();
    }
}
