package com.example.new_test_lab.controller;

import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.model.Rate;
import com.example.new_test_lab.service.RateService;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static com.example.new_test_lab.util.AppUtil.toDto;


@Slf4j
@RestController
@RequestMapping("/rates")
public class RateController {

    private RateService service;

    public RateController(RateService service) {
        this.service = service;
    }

    @ApiOperation( "Получение рейта по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by id: {}", id);
        return new SuccessResponse(service.get(id));
    }

    @ApiOperation( "Получение всех рейтов")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAll() {
        log.info("#get all");

        List<Rate> rates = service.getAll();
        return new SuccessResponse(toDto(rates));
    }

    @ApiOperation( "Создание рейта")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object create(@RequestBody @Valid Rate rate) {
        log.info("#create rate: {}", rate);

        service.create(rate);
        return new SuccessResponse();
    }

    @ApiOperation( "Обновление рейта")
    @PutMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object update(@RequestBody @Valid Rate rate) {
        log.info("#update rate: {}", rate);

        service.update(rate);
        return new SuccessResponse();
    }

    @ApiOperation( "Удаление рейта по ID")
    @DeleteMapping(path = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        service.delete(id);
        return new SuccessResponse();
    }
}
