package com.example.new_test_lab.controller;


import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.model.Section;
import com.example.new_test_lab.service.SectionService;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Slf4j
@RestController
@RequestMapping("/sections")
public class SectionController {

    private SectionService sectionService;

    public SectionController(SectionService sectionService) {
        this.sectionService = sectionService;
    }

    @ApiOperation( "Получение секции по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id)
    {
        log.info("#get by id: {}", id);
        return new SuccessResponse(sectionService.get(id));
    }

    @ApiOperation( "Получение всех секций")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAll() {
        log.info("#get all");

        List<Section> sections = sectionService.getAll();
        return new SuccessResponse(sections);
    }

    @ApiOperation( "Создание секций")
    @PostMapping(produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object create(@RequestBody @Valid Section section) {
        log.info("#create section: {}", section);

        sectionService.create(section);
        return new SuccessResponse();
    }

    @ApiOperation( "Обновление секций")
    @PutMapping(produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object update(@RequestBody @Valid Section section) {
        log.info("#update section: {}", section);

        sectionService.update(section);
        return new SuccessResponse();
    }

    @ApiOperation( "Удаление секции по ID")
    @DeleteMapping(value ="/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        sectionService.delete(id);
        return new SuccessResponse();
    }
}
