package com.example.new_test_lab.controller;


import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.model.SysParams;
import com.example.new_test_lab.repository.SysParamsRepository;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.http.MediaType;
import org.springframework.security.crypto.encrypt.TextEncryptor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@RestController
@RequestMapping("/params")
@RequiredArgsConstructor
public class SysParamsController {

    private SysParamsRepository repository;
    private TextEncryptor encryptor;
    private DefaultListableBeanFactory beanFactory;

    public SysParamsController(SysParamsRepository repository, TextEncryptor encryptor, DefaultListableBeanFactory beanFactory) {
        this.repository = repository;
        this.encryptor = encryptor;
        this.beanFactory = beanFactory;
    }

    @ApiOperation("Получение настроек")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get() {
        log.info("#get");

        return new SuccessResponse(prepareToSend(repository.get()));
    }

    @ApiOperation("Обновление настроек")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object save(@RequestBody @Valid SysParams params) {
        log.info("#save params");

        SysParams paramsDb = repository.get();
        repository.save(prepareToSave(params, paramsDb));
        reloadParameterizableBeans();
        return new SuccessResponse();
    }

    private SysParams prepareToSave(SysParams sysParams, SysParams sysParamsDb) {
        if (StringUtils.isNotEmpty(sysParams.getSmtpPassword())) {
            sysParams.setSmtpPassword(encryptor.encrypt(sysParams.getSmtpPassword()));
        } else sysParams.setSmtpPassword(sysParamsDb.getSmtpPassword());
        if (StringUtils.isNotEmpty(sysParams.getTinkoffPassword())) {
            sysParams.setTinkoffPassword(encryptor.encrypt(sysParams.getTinkoffPassword()));
        } else sysParams.setTinkoffPassword(sysParamsDb.getTinkoffPassword());
        return sysParams;
    }

    private SysParams prepareToSend(SysParams sysParams) {
        sysParams.setSmtpPassword(null);
        sysParams.setTinkoffPassword(null);
        return sysParams;
    }

    private void reloadParameterizableBeans() {
        beanFactory.destroySingleton("emailService");
        beanFactory.destroySingleton("tinkoff");
    }
}
