package com.example.new_test_lab.controller;


import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.*;
import com.example.new_test_lab.model.CandidateAnswer;
import com.example.new_test_lab.model.CandidateQuestion;
import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.service.CandidateQuestionService;
import com.example.new_test_lab.service.CandidateTestService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.AppUtil.isAnsweredCorrect;
import static com.example.new_test_lab.util.SecurityUtil.authUserId;


@Slf4j
@RestController
@RequestMapping("/test-result")
public class TestResultController {

    private CandidateTestService testService;

    private CandidateQuestionService questionService;

    @Autowired
    public TestResultController(CandidateTestService testService, CandidateQuestionService questionService) {
        this.testService = testService;
        this.questionService = questionService;
    }

    @ApiOperation( "Получение результата теста кандидата по коду")
    @GetMapping(value = "/{resultCode}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getResult(@PathVariable String resultCode) {
        log.info("#get by result code: {}", resultCode);

        CandidateTest candidateTest = testService.getByResultCode(resultCode);
        if (candidateTest.isNotViewed()) {
            testService.setViewed(candidateTest, authUserId());
        }
        List<TopicStatDto> topicStats = testService.getTopicStatistic(candidateTest.getId());
        if (candidateTest.getCorrectQuestionNumber() == null)
            candidateTest.setCorrectQuestionNumber(topicStats.stream()
                                                             .map(TopicStatDto::getCorrectQuestions)
                                                             .mapToInt(Integer::intValue)
                                                             .sum());

        Object body;
        List<CandidateQuestion> questions = questionService.getAllQuestionsByResultCode(resultCode);
        if (questions == null) {
            body = new CandidateTestDto(candidateTest, topicStats);
        } else {
            Map<Long, List<CandidateAnswer>> answersMap =
                    questionService.getAnswers(questions.stream()
                                                        .mapToLong(CandidateQuestion::getId)
                                                        .toArray())
                                   .stream()
                                   .collect(Collectors.groupingBy(CandidateAnswer::getCandidateQuestionId));
            List<CandidateQuestionDto> answers = questions.stream()
                                                          .map(question -> new CandidateQuestionDto(
                                                                  question,
                                                                  answersMap.get(question.getId())
                                                                            .stream()
                                                                            .map(ca -> new CandidateAnswerDto(
                                                                                    ca, true
                                                                                 )
                                                                            )
                                                                            .collect(Collectors.toList()),
                                                                  isAnsweredCorrect(answersMap.get(question.getId()),
                                                                                    question.getType())
                                                               )
                                                          )
                                                          .collect(Collectors.toList());
            body = new CandidateTestWithAnswersDto(candidateTest, topicStats, answers);
        }

        return new SuccessResponse(body);
    }

}
