package com.example.new_test_lab.controller;

import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.SectionTopicDto;
import com.example.new_test_lab.model.Section;
import com.example.new_test_lab.model.Topic;
import com.example.new_test_lab.service.SectionService;
import com.example.new_test_lab.service.TopicService;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static com.example.new_test_lab.util.AppUtil.toDto;

@Slf4j
@RestController
@RequestMapping("/topics")
public class TopicController {

    private SectionService sectionService;
    private TopicService topicService;

    public TopicController(SectionService sectionService, TopicService topicService) {
        this.sectionService = sectionService;
        this.topicService = topicService;
    }

    @ApiOperation( "Получение категории по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by id: {}", id);

        return new SuccessResponse(topicService.get(id));
    }

    @ApiOperation( "Получение всех категорий")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAll() {
        log.info("#getAll");

        List<Topic> topics = topicService.getAll();
        List<Section> sections = sectionService.getAll();
        List<SectionTopicDto> list = toDto(sections, topics);
        return new SuccessResponse(list);
    }

    @ApiOperation( "Создание категории")
    @PostMapping(produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object create(@RequestBody @Valid Topic topic) {
        log.info("#update topic: {}", topic);

        topicService.create(topic);
        return new SuccessResponse();
    }

    @ApiOperation( "Обновление категории по ID")
    @PutMapping(produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object update(@RequestBody @Valid Topic topic) {
        log.info("#update topic: {}", topic);

        topicService.create(topic);
        return new SuccessResponse();
    }

    @ApiOperation( "Удаление категории по ID")
    @DeleteMapping(value ="/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        topicService.delete(id);
        return new SuccessResponse();
    }
}
