package com.example.new_test_lab.email;

import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.simplejavamail.api.email.Email;
import org.simplejavamail.api.mailer.AsyncResponse;
import org.simplejavamail.api.mailer.Mailer;
import org.simplejavamail.api.mailer.config.TransportStrategy;
import org.simplejavamail.email.EmailBuilder;
import org.simplejavamail.mailer.MailerBuilder;
import org.springframework.scheduling.annotation.Async;
import org.springframework.security.crypto.encrypt.TextEncryptor;
import org.springframework.stereotype.Service;

import java.util.Objects;

@Slf4j
public class EmailService {

    private static final String CONFIRMATION_CODE_VARIABLE = "[$CONFIRMATION_CODE$]";
    private static final String CONFIRMATION_EMAIL_VARIABLE = "[$CONFIRMATION_EMAIL$]";
    private static final String CONFIRMATION_PASSWORD_VARIABLE = "[$CONFIRMATION_PASSWORD$]";

    private static final String RESET_PASSWORD_VARIABLE = "[$RESET_PASSWORD$]";

    private static final String TEST_ID_VARIABLE = "[$TEST_ID$]";
    private static final String CANDIDATE_NAME_VARIABLE = "[$CANDIDATE_NAME$]";
    private static final String TEST_NAME_VARIABLE = "[$TEST_NAME$]";
    private static final String TEST_RESULT_VARIABLE = "[$TEST_RESULT$]";

    private EmailParams params;
    private TextEncryptor encryptor;
    private Mailer mailer;

    public EmailService(EmailParams params, TextEncryptor encryptor) {
        this.params = params;
        this.encryptor = encryptor;
        checkParams();
        init();
    }

    @Async
    public void sendConfirmation(String email, String password, String confirmationCode) {
        log.info("Sending confirmation code to {}", email);

        String message = StringUtils.replace(params.getEmailRegisterConfirmationMessage(), CONFIRMATION_CODE_VARIABLE, confirmationCode);
        message = StringUtils.replace(message, CONFIRMATION_EMAIL_VARIABLE, email);
        message = StringUtils.replace(message, CONFIRMATION_PASSWORD_VARIABLE, password);
        send(params.getEmailRegisterConfirmationSubject(), message, email);
    }

    @Async
    public void sendResetPassword(String email, String code) {
        log.info("Sending reset password email to {}", email);

        String message = StringUtils.replace(params.getEmailResetPasswordMessage(), RESET_PASSWORD_VARIABLE, code);
        send(params.getEmailResetPasswordSubject(), message, email);
    }

    @Async
    public void sendTestFinished(String email, String candidateName, String testName, String result, String testId) {
        log.info("Sending reset password email to {}", email);

        String message = StringUtils.replace(params.getEmailFinishTestMessage(), TEST_ID_VARIABLE, testId);
        message = StringUtils.replace(message, CANDIDATE_NAME_VARIABLE, candidateName);
        message = StringUtils.replace(message, TEST_NAME_VARIABLE, testName);
        message = StringUtils.replace(message, TEST_RESULT_VARIABLE, result);
        send(params.getEmailFinishTestSubject(), message, email);
    }

    private void checkParams() {
        if (StringUtils.isBlank(params.getSmtpHost()) ||
                StringUtils.isBlank(params.getSmtpUser()) ||
                Objects.isNull(params.getSmtpPort()) ||
                StringUtils.isBlank(params.getSmtpPassword()))
            throw new ServerException(Exceptions.PARAMS_NOT_FOUND);
    }

    private void send(String subject, String message, String emailAddress) {
        Email email = EmailBuilder.startingBlank()
                .to(emailAddress)
                .from(params.getEmailName(), params.getSmtpEmail())
                .withSubject(subject)
                .withPlainText(message)
                .buildEmail();
        AsyncResponse response = mailer.sendMail(email, true);
        response.onException(e -> log.error(e.toString()));
    }

    private void init() {
        mailer = MailerBuilder
                .withSMTPServer(params.getSmtpHost(), params.getSmtpPort(), params.getSmtpUser(), encryptor.decrypt(params.getSmtpPassword()))
                .withTransportStrategy(TransportStrategy.SMTP_TLS)
                .withDebugLogging(true)
                .buildMailer();
    }
}
