package com.example.new_test_lab.model;

import com.example.new_test_lab.jackson.CustomEnumDeserializer;
import com.example.new_test_lab.jackson.CustomEnumSerializer;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.model.enums.Position;
import com.example.new_test_lab.model.enums.QuestionType;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.validation.constraints.Size;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;


@Data
@NoArgsConstructor
@AllArgsConstructor
public class Question implements HasId {

    @Positive(message = "{test.system.error.question.id}")
    private Long id;
    @NotBlank(message = "{test.system.error.question.description}")
    @Size(min = 1, max = 4096, message = "{test.system.error.question.description}")
    private String description;
    @NotNull(message = "{test.system.error.question.topicId}")
    @Positive(message = "{test.system.error.question.topicId}")
    private Long topicId;
    @NotNull(message = "{test.system.error.question.type}")
    @JsonSerialize(using = CustomEnumSerializer.class)
    @JsonDeserialize(using = CustomEnumDeserializer.class)
    private QuestionType type;
    @JsonSerialize(using = CustomEnumSerializer.class)
    @JsonDeserialize(using = CustomEnumDeserializer.class)
    private Level level;
    @JsonSerialize(using = CustomEnumSerializer.class)
    @JsonDeserialize(using = CustomEnumDeserializer.class)
    private Position position;
    @NotNull(message = "{test.system.error.question.complexity}")
    private Boolean increasedComplexity;

    public Question(Question question) {
        this(question.getId(), question.getDescription(), question.getTopicId(), question.getType(), question.getLevel(), question.getPosition(), question.getIncreasedComplexity());
    }
}
