package com.example.new_test_lab.model;

import com.example.new_test_lab.jackson.CustomLocalDateTimeDeserializer;
import com.example.new_test_lab.jackson.CustomLocalDateTimeSerializer;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.proxy.HibernateProxy;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.Objects;

@Getter
@Setter
@ToString
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "result_codes")
public class ResultCode implements HasId {

    @Id
    @SequenceGenerator(name = "result_codes_seq", sequenceName = "result_codes_seq", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "result_codes_seq")
    @Column(name = "id")
    private Long id;

    @Column(name = "code", unique = true, nullable = false)
    @javax.validation.constraints.NotNull
    private String code;

    @NotNull
    @Column(name = "candidate_test_id")
    private Long candidateTestId;

    @JsonSerialize(using = CustomLocalDateTimeSerializer.class)
    @JsonDeserialize(using = CustomLocalDateTimeDeserializer.class)
    @Column(name = "created_at", nullable = false)
    private LocalDateTime createdAt;

    @Column(name = "ttl")
    private Long ttl;

    @Column(name = "can_view_answers")
    private boolean canViewAnswers;

    public ResultCode(ResultCode resultCode) {
        this(resultCode.getId(), resultCode.getCode(), resultCode.getCandidateTestId(), resultCode.getCreatedAt(),
                resultCode.getTtl(), resultCode.isCanViewAnswers());
    }

    @Override
    public final boolean equals(Object o) {
        if (this == o) return true;
        if (o == null) return false;
        Class<?> oEffectiveClass = o instanceof HibernateProxy
                ? ((HibernateProxy) o).getHibernateLazyInitializer().getPersistentClass()
                : o.getClass();
        Class<?> thisEffectiveClass = this instanceof HibernateProxy
                ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass()
                : this.getClass();
        if (thisEffectiveClass != oEffectiveClass) return false;
        ResultCode that = (ResultCode) o;
        return getId() != null && Objects.equals(getId(), that.getId());
    }

    @Override
    public final int hashCode() {
        return this instanceof HibernateProxy
                ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass().hashCode()
                : getClass().hashCode();
    }

}
