package com.example.new_test_lab.model;

import com.example.new_test_lab.jackson.CustomEnumDeserializer;
import com.example.new_test_lab.jackson.CustomEnumSerializer;
import com.example.new_test_lab.jackson.CustomLocalDateTimeDeserializer;
import com.example.new_test_lab.jackson.CustomLocalDateTimeSerializer;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.model.enums.Position;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.validation.constraints.Size;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDateTime;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class Test implements HasId {
    @Positive(message = "{test.system.error.test.id}")
    private Long id;
    @NotBlank(message = "{test.system.error.test.name.empty}")
    @Size(min = 1, max = 256, message = "{test.system.error.test.name.empty}")
    private String name;
    @Size(max = 1024, message = "{test.system.error.test.description.empty}")
    private String description;
    @NotNull(message = "{test.system.error.test.duration}")
    @Positive(message = "{test.system.error.test.duration}")
    private Integer duration;
    private Long userId;
    @JsonSerialize(using = CustomLocalDateTimeSerializer.class)
    @JsonDeserialize(using = CustomLocalDateTimeDeserializer.class)
    private LocalDateTime created = LocalDateTime.now();
    @JsonSerialize(using = CustomEnumSerializer.class)
    @JsonDeserialize(using = CustomEnumDeserializer.class)
    private Level level;
    @JsonSerialize(using = CustomEnumSerializer.class)
    @JsonDeserialize(using = CustomEnumDeserializer.class)
    private Position position;
    @NotNull(message = "{test.system.error.test.questionNumber}")
    @Positive(message = "{test.system.error.test.questionNumber}")
    private Integer questionNumber;
    @NotNull(message = "{test.system.error.test.complexity}")
    private boolean increasedComplexity;

    public Test(Test test) {
        this(test.getId(), test.getName(), test.getDescription(), test.getDuration(), test.getUserId(),
             test.getCreated(), test.getLevel(), test.getPosition(), test.getQuestionNumber(),
             test.isIncreasedComplexity());
    }
}
