package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Account;
import com.example.new_test_lab.repository.AccountRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


@Repository
public class AccountRepositoryImpl implements AccountRepository {
    private static final RowMapper<Account> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Account.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public AccountRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("accounts")
                .usingGeneratedKeyColumns("id");
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    public Account save(Account account) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(account);
        if (jdbcTemplate.queryForObject("SELECT count(1) FROM accounts WHERE user_id = ?", Integer.class, account.getUserId()) == 0) {
            if (insertTest.execute(source) == 0)
                return null;
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE accounts " +
                            "   SET test_balance=:testBalance, user_limit=:userLimit, subscription_start=:subscriptionStart, subscription_end=:subscriptionEnd" +
                            " WHERE user_id=:userId"
                    , source) == 0) {
                return null;
            }
        }
        return account;
    }

    @Override
    public Account get(long userId) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM accounts WHERE user_id=?", ROW_MAPPER, userId));
    }
}
