package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Answer;
import com.example.new_test_lab.repository.AnswerRepository;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.util.List;

@Repository
public class AnswerRepositoryImpl implements AnswerRepository {

    private static final RowMapper<Answer> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Answer.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertAnswer;

    public AnswerRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
        this.insertAnswer = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("answers")
                .usingGeneratedKeyColumns("id");
    }

    @Override
    public List<Answer> getByTestId(long testId) {
        return jdbcTemplate.query("SELECT * FROM answers a JOIN test_questions tq ON a.question_id=tq.question_id WHERE tq.test_id=?", ROW_MAPPER, testId);
    }

    @Override
    public Answer save(Answer answer) {
        BeanPropertySqlParameterSource parameterSource = new BeanPropertySqlParameterSource(answer);

        if (answer.isNew()) {
            Number newKey = insertAnswer.executeAndReturnKey(parameterSource);
            answer.setId(newKey.longValue());
        } else {
            if (namedParameterJdbcTemplate.update(
                    "UPDATE answers SET question_id=:questionId, description=:description, correct=:correct WHERE id=:id", parameterSource) == 0) {
                return null;
            }
        }
        return answer;
    }

    @Override
    public List<Answer> getByQuestionId(long questionId) {
        return jdbcTemplate.query("SELECT * FROM answers WHERE question_id=?", ROW_MAPPER, questionId);
    }

    @Override
    public boolean deleteByQuestionId(long questionId) {
        return jdbcTemplate.update("DELETE FROM answers WHERE question_id=?", questionId) != 0;
    }
}
