package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.CandidateAnswer;
import com.example.new_test_lab.repository.CandidateAnswerRepository;
import com.google.common.primitives.Longs;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.stereotype.Repository;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class CandidateAnswerRepositoryImpl implements CandidateAnswerRepository {
    private static final RowMapper<CandidateAnswer> ROW_MAPPER = BeanPropertyRowMapper.newInstance(CandidateAnswer.class);

    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    public CandidateAnswerRepositoryImpl(NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    public int[] saveAll(List<CandidateAnswer> answers) {
        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(answers.toArray());
        int[] updateCounts = answers.stream().anyMatch(CandidateAnswer::isNew) ?
                namedParameterJdbcTemplate.batchUpdate(
                        "INSERT INTO candidate_answers(candidate_question_id, description, correct) VALUES (:candidateQuestionId, :description, :correct)", batch) :
                namedParameterJdbcTemplate.batchUpdate(
                        "UPDATE candidate_answers SET description=:description, correct=:correct, selected=:selected WHERE" +
                                " candidate_question_id=:candidateQuestionId AND id=:id", batch);
        return updateCounts;
    }

    @Override
    public List<CandidateAnswer> getByQuestionId(long... questionId) {
        SqlParameterSource parameters = new MapSqlParameterSource("ids", Longs.asList(questionId));
        return namedParameterJdbcTemplate.query("SELECT * FROM candidate_answers WHERE candidate_question_id IN (:ids)", parameters, ROW_MAPPER);
    }

    @Override
    public CandidateAnswer get(long id, long questionId) {
        Map<String, Object> params = new HashMap<>();
        params.put("id", id);
        params.put("questionId", questionId);
        SqlParameterSource parameters = new MapSqlParameterSource(params);
        return DataAccessUtils.singleResult(namedParameterJdbcTemplate.query("SELECT * FROM candidate_answers WHERE candidate_question_id=:questionId AND id =:id", parameters, ROW_MAPPER));
    }
}
