package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.CandidateQuestion;
import com.example.new_test_lab.repository.CandidateQuestionRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.sql.Types;
import java.util.List;

@Repository
public class CandidateQuestionRepositoryImpl implements CandidateQuestionRepository {
    private static final RowMapper<CandidateQuestion> ROW_MAPPER = BeanPropertyRowMapper.newInstance(CandidateQuestion.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public CandidateQuestionRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("candidate_questions")
                .usingGeneratedKeyColumns("id");
    }

    @Override
    public CandidateQuestion save(CandidateQuestion question) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(question);
        source.registerSqlType("type", Types.VARCHAR);

        if (question.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            question.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE candidate_questions " +
                            " SET description=:description, candidate_test_id=:candidateTestId, " +
                            "topic_name=:topicName, type=:type, answered=:answered" +
                            " WHERE id=:id"
                    , source) == 0) {
                return null;
            }
        }
        return question;
    }

    @Override
    public CandidateQuestion get(long testId, long questionId) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM candidate_questions WHERE candidate_test_id =? AND id =?", ROW_MAPPER, testId, questionId));

    }

    @Override
    public List<CandidateQuestion> getByTestId(long candidateTestId) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("testId", candidateTestId);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM candidate_questions cq WHERE cq.candidate_test_id=:testId ORDER BY topic_name", parameters, ROW_MAPPER);
    }

    @Override
    public List<CandidateQuestion> getByTestId(long candidateTestId, int page, int size) {
        int offset = page * size;

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("testId", candidateTestId);
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM candidate_questions cq WHERE cq.candidate_test_id=:testId ORDER BY topic_name, id OFFSET :offset LIMIT :size", parameters, ROW_MAPPER);
    }
}
