package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Rate;
import com.example.new_test_lab.repository.RateRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.util.List;

@Repository
public class RateRepositoryImpl implements RateRepository {
    private static final RowMapper<Rate> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Rate.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public RateRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("rates")
                .usingGeneratedKeyColumns("id");
    }

    @Override
    public Rate save(Rate rate) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(rate);
        if (rate.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            rate.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE rates " +
                            " SET name=:name, bill_price=:billPrice, test_amount=:testAmount, subscription_days=:subscriptionDays" +
                            " WHERE id=:id"
                    , source) == 0) {
                return null;
            }
        }
        return rate;
    }

    @Override
    public List<Rate> getAll() {
        return jdbcTemplate.query("SELECT * FROM rates", ROW_MAPPER);
    }

    @Override
    public Rate get(long id) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM rates WHERE id = " + id, ROW_MAPPER));
    }

    @Override
    public boolean delete(long id) {
        return jdbcTemplate.update("DELETE FROM rates WHERE id=?", id) != 0;
    }

}
