package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Section;
import com.example.new_test_lab.repository.SectionRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.util.List;

@Repository
public class SectionRepositoryImpl implements SectionRepository {
    private static final RowMapper<Section> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Section.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public SectionRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("sections")
                .usingGeneratedKeyColumns("id");
    }

    @Override
    public Section save(Section section) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(section);
        if (section.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            section.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE sections " +
                            "   SET name=:name" +
                            " WHERE id=:id"
                    , source) == 0) {
                return null;
            }
        }
        return section;
    }

    @Override
    public List<Section> getAll() {
        return jdbcTemplate.query("SELECT * FROM sections", ROW_MAPPER);
    }

    @Override
    public Section get(long id) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM sections WHERE id = " + id, ROW_MAPPER));
    }

    @Override
    public boolean delete(long id) {
        return jdbcTemplate.update("DELETE FROM sections WHERE id=?", id) != 0;
    }
}
