package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.SysParams;
import com.example.new_test_lab.repository.SysParamsRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;


@Repository
@Transactional(readOnly = true)
public class SysParamsRepositoryImpl implements SysParamsRepository {

    private static final RowMapper<SysParams> ROW_MAPPER = BeanPropertyRowMapper.newInstance(SysParams.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    public SysParamsRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    @Transactional
    public SysParams save(SysParams params) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(params);
        if (namedParameterJdbcTemplate.update("" +
                        "UPDATE sys_params " +
                        "   SET smtp_user=:smtpUser, " +
                        "smtp_password=:smtpPassword, " +
                        "smtp_host=:smtpHost, " +
                        "smtp_port=:smtpPort, " +
                        "smtp_email=:smtpEmail, " +
                        "email_name=:emailName, " +
                        "email_register_confirmation_subject=:emailRegisterConfirmationSubject, " +
                        "email_register_confirmation_message=:emailRegisterConfirmationMessage, " +
                        "email_reset_password_subject=:emailResetPasswordSubject, " +
                        "email_reset_password_message=:emailResetPasswordMessage, " +
                        "email_finish_test_subject=:emailFinishTestSubject, " +
                        "email_finish_test_message=:emailFinishTestMessage, " +
                        "default_test_balance=:defaultTestBalance, " +
                        "default_subscription_days=:defaultSubscriptionDays, " +
                        "default_user_limit=:defaultUserLimit, " +
                        "tinkoff_url=:tinkoffUrl, " +
                        "tinkoff_terminal_key=:tinkoffTerminalKey, " +
                        "tinkoff_password=:tinkoffPassword, " +
                        "temp_invoice_store_path=:tempInvoiceStorePath " +
                        " WHERE sys_params_id=(SELECT sys_params_id FROM sys_params LIMIT 1)"
                , source) == 0) {
            return null;
        }

        return params;
    }

    @Override
    public SysParams get() {
        return  DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM sys_params LIMIT 1", ROW_MAPPER));
    }
}
