package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Topic;
import com.example.new_test_lab.repository.TopicRepository;
import com.google.common.primitives.Longs;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.util.List;

@Repository
public class TopicRepositoryImpl implements TopicRepository {
    private static final RowMapper<Topic> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Topic.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public TopicRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("topics")
                .usingGeneratedKeyColumns("id");
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    public Topic save(Topic topic) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(topic);
        if (topic.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            topic.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE topics " +
                            "   SET name=:name, section_id=:sectionId" +
                            " WHERE id=:id"
                    , source) == 0) {
                return null;
            }
        }
        return topic;
    }

    @Override
    public List<Topic> getAll() {
        return jdbcTemplate.query("SELECT * FROM topics", ROW_MAPPER);
    }

    @Override
    public Topic get(long id) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM topics WHERE id = " + id, ROW_MAPPER));
    }

    @Override
    public List<Topic> getByIds(long... ids) {
        SqlParameterSource parameters = new MapSqlParameterSource("ids", Longs.asList(ids));
        return namedParameterJdbcTemplate.query("SELECT * FROM topics WHERE id IN (:ids)", parameters, ROW_MAPPER);
    }

    @Override
    public boolean delete(long id) {
        return jdbcTemplate.update("DELETE FROM topics WHERE id=?", id) != 0;
    }
}
