package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.model.Rate;
import com.example.new_test_lab.repository.RateRepository;
import com.example.new_test_lab.service.RateService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;

import static com.example.new_test_lab.util.ValidationUtil.checkNew;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;


@Slf4j
@Service
@Transactional(readOnly = true)
public class RateServiceImpl implements RateService {

    private RateRepository repository;

    @Autowired
    public RateServiceImpl(RateRepository repository) {
        this.repository = repository;
    }

    @Override
    @Cacheable(value = "rates")
    public List<Rate> getAll() {
        log.info("Getting all");

        return repository.getAll();
    }

    @Override
    public Rate get(long id) {
        log.info("Getting by id={}", id);

        return checkNotFoundWithId(repository.get(id), id);
    }

    @Override
    @Transactional
    @CacheEvict(value = "rates", allEntries = true)
    public Rate create(Rate rate) {
        log.info("Creating rate: {}", repository);

        checkNew(rate, true);
        return repository.save(rate);
    }

    @Override
    @Transactional
    @CacheEvict(value = "rates", allEntries = true)
    public void update(Rate rate) {
        log.info("Updating rate: {}", rate);

        checkNew(rate, false);

        checkNotFoundWithId(repository.save(rate), rate.getId());
    }

    @Override
    @Transactional
    @CacheEvict(value = "rates", allEntries = true)
    public void delete(long id) {
        log.info("Deleting by id = {}", id);

        checkNotFoundWithId(repository.delete(id), id);
    }
}
