package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.model.Section;
import com.example.new_test_lab.repository.SectionRepository;
import com.example.new_test_lab.service.SectionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static com.example.new_test_lab.util.ValidationUtil.checkNew;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;


@Slf4j
@Service
@Transactional(readOnly = true)
public class SectionServiceImpl implements SectionService {

    private SectionRepository repository;

    @Autowired
    public SectionServiceImpl(SectionRepository repository) {
        this.repository = repository;
    }

    @Override
    @Cacheable(value = "sections")
    public List<Section> getAll() {
        log.info("Get all sections");
        return repository.getAll();
    }

    @Override
    public Section get(long id) {
        log.info("Get section with id = {}", id);
        return checkNotFoundWithId(repository.get(id), id);
    }

    @Override
    @Transactional
    @CacheEvict(value = "sections", allEntries = true)
    public Section create(Section section) {
        log.info("Creating section: {}", section);

        checkNew(section, true);
        return repository.save(section);
    }

    @Override
    @Transactional
    @CacheEvict(value = "sections", allEntries = true)
    public void update(Section section) {
        log.info("Updating section: {}", section);

        checkNew(section, false);
        checkNotFoundWithId(repository.save(section), section.getId());
    }

    @Override
    @Transactional
    @CacheEvict(value = "sections", allEntries = true)
    public void delete(long id) {
        log.info("Deleting by section id = {}", id);
        checkNotFoundWithId(repository.delete(id), id);
    }
}
