package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.model.Topic;
import com.example.new_test_lab.repository.TopicRepository;
import com.example.new_test_lab.service.TopicService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;

import static com.example.new_test_lab.util.ValidationUtil.checkNew;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;


@Slf4j
@Service
@Transactional(readOnly = true)
@AllArgsConstructor
public class TopicServiceImpl implements TopicService {

    private TopicRepository repository;

    @Override
    @Cacheable(value = "topics")
    public List<Topic> getAll() {
        log.info("Get all topics");
        return repository.getAll();
    }

    @Override
    public Topic get(long id) {
        log.info("Get topic with id = {}", id);
        return checkNotFoundWithId(repository.get(id), id);
    }

    @Override
    @Transactional
    @CacheEvict(value = "topics", allEntries = true)
    public Topic create(Topic topic) {
        log.info("Creating new topic: {}", topic);

        checkNew(topic, true);
        return repository.save(topic);
    }

    @Override
    @Transactional
    @CacheEvict(value = "topics", allEntries = true)
    public void update(Topic topic) {
        log.info("Updating topic: {}", topic);

        checkNew(topic, false);
        checkNotFoundWithId(repository.save(topic), topic.getId());
    }

    @Override
    @Transactional
    @CacheEvict(value = "topics", allEntries = true)
    public void delete(long id) {
        log.info("Deleting by topic id = {}", id);

        checkNotFoundWithId(repository.delete(id), id);
    }
}
