package com.example.new_test_lab.tinkoff;


import com.example.new_test_lab.tinkoff.request.BaseRequest;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

public class TinkoffSignUtil {

    public static String getSign(BaseRequest request) throws NoSuchAlgorithmException {
        StringBuilder signBuilder = new StringBuilder();
        getFields(request).forEach((k, v) -> signBuilder.append(v.toString()));
        return encode(signBuilder.toString());
    }

    public static boolean validateSign(BaseRequest request, String password) throws NoSuchAlgorithmException {
        Map<String, Object> fieldValues = getFields(request);
        Object token = fieldValues.remove("token");
        if (Objects.isNull(token)) return false;
        fieldValues.put("password", password);
        StringBuilder signBuilder = new StringBuilder();
        fieldValues.forEach((k, v) -> signBuilder.append(v.toString()));
        String encoded = encode(signBuilder.toString());
        return encoded.equals(token.toString());
    }

    private static <T> Map<String, Object> getFields(T t) {
        Map<String, Object> keyValues = new TreeMap<>(Comparator.naturalOrder());
        Class clazz = t.getClass();
        while (clazz != Object.class) {
            Arrays.asList(clazz.getDeclaredFields())
                    .forEach(f -> {
                        try {
                            f.setAccessible(true);
                            Object value = f.get(t);
                            if (Objects.nonNull(value)) {
                                keyValues.put(f.getName(), value);
                            }
                        } catch (IllegalAccessException e) {
                        }
                    });
            clazz = clazz.getSuperclass();
        }

        return keyValues;
    }

    private static String encode(String toEncode) throws NoSuchAlgorithmException {
        MessageDigest digest = MessageDigest.getInstance("SHA-256");
        byte[] hash = digest.digest(toEncode.getBytes(StandardCharsets.UTF_8));
        return bytesToHex(hash);
    }

    private static String bytesToHex(byte[] hash) {
        StringBuffer hexString = new StringBuffer();
        for (int i = 0; i < hash.length; i++) {
            String hex = Integer.toHexString(0xff & hash[i]);
            if(hex.length() == 1) {
                hexString.append('0');
            }
            hexString.append(hex);
        }
        return hexString.toString();
    }
}
