package com.example.new_test_lab.util;

import com.example.new_test_lab.model.Order;
import com.example.new_test_lab.model.Rate;
import com.example.new_test_lab.model.enums.OrderStatus;
import com.example.new_test_lab.model.enums.PaymentMethod;
import com.google.common.base.Preconditions;


import java.math.BigDecimal;

public class OrderUtil {

    public static Order createNewOrder(Rate rate, long userId, PaymentMethod paymentMethod) {
        Order order = new Order();
        order.setAmount(rate.getBillPrice());
        order.setRateId(rate.getId());
        order.setSubscriptionDays(rate.getSubscriptionDays());
        order.setUserId(userId);
        order.setTestQuantity(rate.getTestAmount());
        order.setPaymentMethod(paymentMethod);
        order.setOrderStatus(OrderStatus.NEW);
        return order;
    }

    public static int getAmountInSmallestUnit(Double amount) {
        Preconditions.checkNotNull(amount);
        return BigDecimal.valueOf(amount)
                .movePointRight(2)
                .intValueExact();
    }

    public static OrderStatus resolveStatus(String tinkoffStatus) {
        switch (tinkoffStatus) {
            case "CONFIRMED":
                return OrderStatus.CONFIRMED;
            case "REVERSED":
            case "REFUNDED":
            case "REJECTED":
            case "DEADLINE_EXPIRED":
            case "CANCELED":
                return OrderStatus.CANCELED;
            default:
                return OrderStatus.PENDING;
        }

    }
}
