package com.example.new_test_lab.util;

import com.example.new_test_lab.AuthorizedUser;
import com.example.new_test_lab.model.User;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;


import static java.util.Objects.requireNonNull;

public class SecurityUtil {

    private SecurityUtil() {
    }

    public static AuthorizedUser safeGet() {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        if (auth == null) {
            return null;
        }
        Object principal = auth.getPrincipal();
        return (principal instanceof AuthorizedUser) ? (AuthorizedUser) principal : null;
    }

    public static AuthorizedUser get() {
        AuthorizedUser user = safeGet();
        requireNonNull(user, "No authorized user found");
        return user;
    }

    public static Long authUserId() {
        AuthorizedUser user = safeGet();
        return user != null ? user.getId() : 0;
    }

    public static User authUser() {
        AuthorizedUser user = safeGet();
        return user != null ? user.getUser() : null;
    }
}
