package com.example.new_test_lab.util;

import com.example.new_test_lab.dto.UserDto;
import com.example.new_test_lab.model.User;
import org.springframework.util.StringUtils;


import java.time.LocalDateTime;
import java.util.Objects;

import static com.example.new_test_lab.util.AppUtil.getRandomCode;


public class UserUtil {

    public final static int CONFIRMATION_CODE_LENGTH = 24;

    public static UserDto toDto(User user) {
        return new UserDto(user.getId(), user.getEmail(), user.getPhone(), user.isEnabled(),
                user.getRegistered(), null, user.isConfirmed(), user.getRoles(), user.getFirstName(), user.getLastName(), user.getMiddleName(), user.getCompanyInfoId());
    }

    public static User fromDto(UserDto userDto) {
        return fromDto(userDto, new User());
    }

    public static User fromDto(UserDto userDto, User user) {
        user.setEmail(userDto.getEmail().toLowerCase());
        user.setCompanyInfoId(userDto.getCompanyInfoId());
        user.setPhone(StringUtils.hasText(userDto.getPhone()) ? userDto.getPhone() : null);
        user.setFirstName(userDto.getFirstName());
        user.setLastName(userDto.getLastName());
        user.setMiddleName(userDto.getMiddleName());
        if (StringUtils.hasText(userDto.getPassword())) {
            user.setPassword(userDto.getPassword());
        }
        if (Objects.nonNull(userDto.getEnabled()))
            user.setEnabled(userDto.getEnabled());
        return user;
    }

    public static void fillNewUser(User user) {
        user.setEnabled(true);
        user.setRegistered(LocalDateTime.now());
        String confirmationCode = getRandomCode(CONFIRMATION_CODE_LENGTH);
        user.setConfirmationCode(confirmationCode);
    }
}
