import React from 'react';
require('../theme-setup.less');
import {Router, Route, Switch} from 'react-router-dom';
import { connect } from 'react-redux';
import { history } from '../helpers/history';
import { store } from '../helpers/store';

import { alertActions } from '../actions/alert.actions';
import { PrivateRoute } from '../routes/PrivateRoute';
import { Login } from '../components/Login';
import { SideBarLayout } from '../layouts/SideBarLayout';
import {Alert} from "rsuite";
import {alertConstants} from "../constants/alert.constants";
import loyaltyLog from "../helpers/loyaltyLog";
import {Register} from "../components/Register";
import {PasswordReset} from "../components/PasswordReset";
import {pathConstants} from "../constants/path.constants";
import {PasswordResetConfirm} from "../components/PasswordResetConfirm";

class App extends React.Component {

    constructor(props) {
        super(props);
        store.dispatch(alertActions.clear());
    }

    componentDidUpdate(prevProps, prevState) {
        //TODO alerts force to rerender all route tree
        if (this.props.alert.type === alertConstants.SUCCESS) {
            Alert.success(this.props.alert.message, this.props.alert.duration ? this.props.alert.duration : 5000);
            store.dispatch(alertActions.clear());
        }
        if (this.props.alert.type === alertConstants.ERROR) {
            Alert.error(this.props.alert.message, this.props.alert.duration ? this.props.alert.duration : 5000);
            store.dispatch(alertActions.clear());
        }
    }

    render() {
        loyaltyLog("RENDER COMPONENT APP");

        return (
            <Router history={history}>
                <Switch>
                    <Route exact path={pathConstants.LOGIN} component={Login} />
                    <Route exact path={pathConstants.REGISTER} component={Register} />
                    <Route exact path={pathConstants.PASSWORD_RESET} component={PasswordReset} />
                    <Route exact path={pathConstants.PASSWORD_RESET_CONFIRM} component={PasswordResetConfirm} />
                    <PrivateRoute path="/" comp={SideBarLayout} />
                </Switch>
            </Router>
        );
    }
}

function mapState(state) {
    return {
        alert: state.alert,
        authentication: state.authentication,
        user: state.authentication.user
    };
}

const actionCreators = {
    clearAlerts: alertActions.clear
};

const connectedApp = connect(mapState, actionCreators)(App);
export { connectedApp as App };