import {alertActions} from "./alert.actions";
import {responseMessages} from "../content/responseMessages";
import {history} from "../helpers/history";
import {orderConstants} from "../constants/orders.constants";
import {orderService} from "../services/order.service";
import {pathConstants} from "../constants/path.constants";

export const orderActions = {
    getOrders,
    createOrder,
    requestDocuments
};

function getOrders(page, size, sort, sortType) {

    return dispatch => {
        dispatch(request());

        orderService.getOrders(page, size, sort, sortType)
            .then(
                orders => {
                    dispatch(success(orders));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: orderConstants.GET_ORDERS_REQUEST } }
    function success(orders) { return { type: orderConstants.GET_ORDERS_SUCCESS, orders } }
    function failure(error) { return { type: orderConstants.GET_ORDERS_FAILURE, error } }
}

function createOrder(orderModel) {

    return dispatch => {
        dispatch(request());

        orderService.createOrder(orderModel)
            .then(
                order => {
                    dispatch(success(order));
                    dispatch(alertActions.success(responseMessages.ORDER_CREATED_SUCCESSFULLY));
                    history.push(pathConstants.BILLS);
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: orderConstants.CREATE_ORDERS_REQUEST } }
    function success(order) { return { type: orderConstants.CREATE_ORDERS_SUCCESS, order } }
    function failure(error) { return { type: orderConstants.CREATE_ORDERS_FAILURE, error } }
}

function requestDocuments(orderId, documentsPostName, documentsPostAddress) {

    return dispatch => {
        dispatch(request());

        orderService.requestDocuments(orderId, documentsPostName, documentsPostAddress)
            .then(
                order => {
                    dispatch(success());
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: orderConstants.REQUEST_DOCUMENTS_REQUEST } }
    function success() { return { type: orderConstants.REQUEST_DOCUMENTS_SUCCESS } }
    function failure(error) { return { type: orderConstants.REQUEST_DOCUMENTS_FAILURE, error } }
}