import {alertActions} from "./alert.actions";
import {questionService} from "../services/question.service";
import {questionConstants} from "../constants/question.constants";
import {responseMessages} from "../content/responseMessages";
import {history} from "../helpers/history";

export const questionActions = {
    getQuestions,
    createQuestion,
    updateQuestion,
    deleteQuestion
};

function getQuestions(page, size, sort, sortType) {

    return dispatch => {
        dispatch(request());

        questionService.getQuestions(page, size, sort, sortType)
            .then(
                questions => {
                    dispatch(success(questions));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: questionConstants.GET_QUESTIONS_REQUEST} }
    function success(questions) { return { type: questionConstants.GET_QUESTIONS_SUCCESS, questions } }
    function failure(error) { return { type: questionConstants.GET_QUESTION_BY_ID_FAILURE, error } }
}

function createQuestion(questionModel) {

    return dispatch => {
        dispatch(request());

        questionService.createQuestion(questionModel)
            .then(
                question => {
                    dispatch(success(question));
                    dispatch(alertActions.success(responseMessages.QUESTION_CREATED_SUCCESSFULLY));
                    history.goBack();
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: questionConstants.CREATE_QUESTION_REQUEST} }
    function success(question) { return { type: questionConstants.CREATE_QUESTION_SUCCESS, question } }
    function failure(error) { return { type: questionConstants.CREATE_QUESTION_FAILURE, error } }
}

function updateQuestion(questionModel) {

    return dispatch => {
        dispatch(request());

        questionService.updateQuestion(questionModel)
            .then(
                question => {
                    dispatch(success(question));
                    dispatch(alertActions.success(responseMessages.QUESTION_UPDATED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: questionConstants.UPDATE_QUESTION_REQUEST} }
    function success(question) { return { type: questionConstants.UPDATE_QUESTION_SUCCESS, question } }
    function failure(error) { return { type: questionConstants.UPDATE_QUESTION_SUCCESS, error } }
}

function deleteQuestion(id) {

    return dispatch => {
        dispatch(request());

        questionService.deleteQuestion(id)
            .then(
                question => {
                    dispatch(success(id));
                    dispatch(alertActions.success(responseMessages.QUESTION_DELETED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: questionConstants.DELETE_QUESTION_REQUEST} }
    function success(questionId) { return { type: questionConstants.DELETE_QUESTION_SUCCESS, questionId } }
    function failure(error) { return { type: questionConstants.DELETE_QUESTION_FAILURE, error } }
}