import {alertActions} from "./alert.actions";
import {sectionService} from "../services/section.service";
import {sectionConstants} from "../constants/section.constants";

export const sectionActions = {
    getSections,
    createSection,
    updateSection,
    deleteSection
};

function getSections(page, size, sort, sortType) {

    return dispatch => {
        dispatch(request());

        sectionService.getSections(page, size, sort, sortType)
            .then(
                sections => {
                    dispatch(success(sections));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: sectionConstants.GET_SECTIONS_REQUEST} }
    function success(sections) { return { type: sectionConstants.GET_SECTIONS_SUCCESS, sections } }
    function failure(error) { return { type: sectionConstants.GET_SECTION_BY_ID_FAILURE, error } }
}

function createSection(sectionModel) {

    return dispatch => {
        dispatch(request());

        sectionService.createSection(sectionModel)
            .then(
                section => {
                    dispatch(success(section));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: sectionConstants.CREATE_SECTION_REQUEST} }
    function success(section) { return { type: sectionConstants.CREATE_SECTION_SUCCESS, section } }
    function failure(error) { return { type: sectionConstants.CREATE_SECTION_FAILURE, error } }
}

function updateSection(sectionModel) {

    return dispatch => {
        dispatch(request());

        sectionService.updateSection(sectionModel)
            .then(
                section => {
                    dispatch(success(section));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: sectionConstants.UPDATE_SECTION_REQUEST} }
    function success(section) { return { type: sectionConstants.UPDATE_SECTION_SUCCESS, section } }
    function failure(error) { return { type: sectionConstants.UPDATE_SECTION_SUCCESS, error } }
}

function deleteSection(id) {

    return dispatch => {
        dispatch(request());

        sectionService.deleteSection(id)
            .then(
                section => {
                    dispatch(success(section));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: sectionConstants.DELETE_SECTION_REQUEST} }
    function success(section) { return { type: sectionConstants.DELETE_SECTION_SUCCESS, section } }
    function failure(error) { return { type: sectionConstants.DELETE_SECTION_FAILURE, error } }
}