import {testService} from "../services/test.service";
import {testConstants} from "../constants/test.constants";
import {alertActions} from "./alert.actions";
import {responseMessages} from "../content/responseMessages";
import {pathConstants} from "../constants/path.constants";
import {history} from "../helpers/history";
import {language} from "../content/language";
import {copyToClipBoard, getTestUrlCode} from "../helpers/util";

export const testActions = {
    getTests,
    createTest,
    deleteTest,
    updateTest,
    createCandidateTest,
    getCandidateTests,
    countQuestionsByTopics
};

function getTests(page, size, sort, sortType) {

    return dispatch => {
        dispatch(request());

        testService.getTests(page, size, sort, sortType)
            .then(
                tests => {
                    dispatch(success(tests));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.GET_TESTS_REQUEST } }
    function success(tests) { return { type: testConstants.GET_TESTS_SUCCESS, tests } }
    function failure(error) { return { type: testConstants.GET_TESTS_FAILURE, error } }
}

function getCandidateTests(page, size, sort, sortType) {

    return dispatch => {
        dispatch(request());

        testService.getCandidateTests(page, size, sort, sortType)
            .then(
                candidateTests => {
                    dispatch(success(candidateTests));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.GET_CANDIDATE_TESTS_REQUEST } }
    function success(candidateTests) { return { type: testConstants.GET_CANDIDATE_TESTS_SUCCESS, candidateTests } }
    function failure(error) { return { type: testConstants.GET_CANDIDATE_TESTS_FAILURE, error } }
}

function createTest(testModel) {

    return dispatch => {
        dispatch(request());

        testService.createTest(testModel)
            .then(
                test => {
                    dispatch(success(test));
                    dispatch(alertActions.success(responseMessages.TEST_CREATED_SUCCESSFULLY));
                    history.goBack();
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.CREATE_TEST_REQUEST } }
    function success(test) { return { type: testConstants.CREATE_TEST_SUCCESS, test } }
    function failure(error) { return { type: testConstants.CREATE_TEST_FAILURE, error } }
}

function createCandidateTest(testId, candidateName, onSuccess) {

    return dispatch => {
        dispatch(request());

        testService.createCandidateTest(testId, candidateName)
            .then(
                test => {
                    dispatch(success(test));
                    dispatch(alertActions.success(language.CANDIDATE_TEST_CREATED_SUCCESSFULLY, 10000));
                    copyToClipBoard(getTestUrlCode(test.data.linkCode), true, dispatch, language.TEST_LINK_CODE_COPIED, 20000);
                    onSuccess();
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.CREATE_CANDIDATE_TEST_REQUEST } }
    function success(test) { return { type: testConstants.CREATE_CANDIDATE_TEST_SUCCESS, test } }
    function failure(error) { return { type: testConstants.CREATE_CANDIDATE_TEST_FAILURE, error } }
}

function updateTest(testModel) {

    return dispatch => {
        dispatch(request());

        testService.updateTest(testModel)
            .then(
                test => {
                    dispatch(success(test));
                    dispatch(alertActions.success(responseMessages.TEST_UPDATED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.UPDATE_TEST_REQUEST } }
    function success(test) { return { type: testConstants.UPDATE_TEST_SUCCESS, test } }
    function failure(error) { return { type: testConstants.UPDATE_TEST_FAILURE, error } }
}

function deleteTest(id) {

    return dispatch => {
        dispatch(request());

        testService.deleteTest(id)
            .then(
                test => {
                    dispatch(success(id));
                    dispatch(alertActions.success(responseMessages.TEST_DELETED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.DELETE_TEST_REQUEST } }
    function success(testId) { return { type: testConstants.DELETE_TEST_SUCCESS, testId } }
    function failure(error) { return { type: testConstants.DELETE_TEST_FAILURE, error } }
}

function countQuestionsByTopics(topicIds) {

    return dispatch => {
        dispatch(request());

        testService.countQuestionsByTopics(topicIds)
            .then(
                countQuestions => {
                    dispatch(success(countQuestions));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: testConstants.COUNT_QUESTIONS_REQUEST } }
    function success(countQuestions) { return { type: testConstants.COUNT_QUESTIONS_SUCCESS, countQuestions } }
    function failure(error) { return { type: testConstants.COUNT_QUESTIONS_FAILURE, error } }
}
