import {alertActions} from "./alert.actions";
import {topicService} from "../services/topic.service";
import {topicConstants} from "../constants/topic.constants";

export const topicActions = {
    getTopics,
    createTopic,
    updateTopic,
    deleteTopic
};

function getTopics(page, size, sort, sortType) {

    return dispatch => {
        dispatch(request());

        topicService.getTopics(page, size, sort, sortType)
            .then(
                topics => {
                    dispatch(success(topics));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: topicConstants.GET_TOPICS_REQUEST} }
    function success(topics) { return { type: topicConstants.GET_TOPICS_SUCCESS, topics } }
    function failure(error) { return { type: topicConstants.GET_TOPIC_BY_ID_FAILURE, error } }
}

function createTopic(topicModel) {

    return dispatch => {
        dispatch(request());

        topicService.createTopic(topicModel)
            .then(
                topic => {
                    dispatch(success(topic));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: topicConstants.CREATE_TOPIC_REQUEST} }
    function success(topic) { return { type: topicConstants.CREATE_TOPIC_SUCCESS, topic } }
    function failure(error) { return { type: topicConstants.CREATE_TOPIC_FAILURE, error } }
}

function updateTopic(topicModel) {

    return dispatch => {
        dispatch(request());

        topicService.updateTopic(topicModel)
            .then(
                topic => {
                    dispatch(success(topic));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: topicConstants.UPDATE_TOPIC_REQUEST} }
    function success(topic) { return { type: topicConstants.UPDATE_TOPIC_SUCCESS, topic } }
    function failure(error) { return { type: topicConstants.UPDATE_TOPIC_SUCCESS, error } }
}

function deleteTopic(id) {

    return dispatch => {
        dispatch(request());

        topicService.deleteTopic(id)
            .then(
                topic => {
                    dispatch(success(topic));
                },
                error => {
                    dispatch(failure(error));
                    dispatch(alertActions.error(error))
                }
            );
    };
    function request() { return { type: topicConstants.DELETE_TOPIC_REQUEST} }
    function success(topic) { return { type: topicConstants.DELETE_TOPIC_SUCCESS, topic } }
    function failure(error) { return { type: topicConstants.DELETE_TOPIC_FAILURE, error } }
}