import { userConstants } from '../constants/user.constants';
import { userService } from '../services/user.service';
import { alertActions } from './alert.actions';
import { history } from '../helpers/history';
import {responseMessages} from "../content/responseMessages";

export const userActions = {
    login,
    register,
    registerConfirm,
    logout,
    updateProfile,
    getRelatedUsers,
    createRelatedUser,
    deleteRelatedUser,
    resetPassword,
    resetPasswordConfirm,
    updateCompanyInfo,
    getCompanyInfo
};

function login(username, password, confirm) {
    return dispatch => {
        dispatch(request({ username }));

        userService.login(username, password)
            .then(
                user => {
                    if (confirm) {
                        dispatch(userActions.registerConfirm(confirm));
                    }
                    dispatch(success(user));
                    history.push('/tests');
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request(user) { return { type: userConstants.LOGIN_REQUEST, user } }
    function success(user) { return { type: userConstants.LOGIN_SUCCESS, user } }
    function failure(error) { return { type: userConstants.LOGIN_FAILURE, error } }
}

function resetPassword(username) {
    return dispatch => {
        dispatch(request({ username }));

        userService.resetPassword(username)
            .then(
                user => {
                    dispatch(success(user));
                    dispatch(alertActions.success(responseMessages.RESET_PASSWORD_SUCCESS, 15000));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request(user) { return { type: userConstants.RESET_PASSWORD_REQUEST, user } }
    function success(user) { return { type: userConstants.RESET_PASSWORD_SUCCESS, user } }
    function failure(error) { return { type: userConstants.RESET_PASSWORD_FAILURE, error } }
}

function resetPasswordConfirm(resetCode, password) {
    return dispatch => {
        dispatch(request());

        userService.resetPasswordConfirm(resetCode, password)
            .then(
                response => {
                    dispatch(success(response));
                    dispatch(alertActions.success(responseMessages.RESET_PASSWORD_CONFIRM, 15000));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request(user) { return { type: userConstants.RESET_PASSWORD_CONFIRM_REQUEST, user } }
    function success(user) { return { type: userConstants.RESET_PASSWORD_CONFIRM_SUCCESS, user } }
    function failure(error) { return { type: userConstants.RESET_PASSWORD_CONFIRM_FAILURE, error } }
}

function registerConfirm(confirm) {
    return dispatch => {
        dispatch(request(confirm));

        userService.registerConfirm(confirm)
            .then(
                user => {
                    dispatch(success(confirm));
                    dispatch(alertActions.success(responseMessages.REGISTER_CONFIRM_SUCCESS, 10000));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request(user) { return { type: userConstants.REG_CONFIRM_REQUEST, user } }
    function success(user) { return { type: userConstants.REG_CONFIRM_SUCCESS, user } }
    function failure(error) { return { type: userConstants.REG_CONFIRM_FAILURE, error } }
}

function register(username, password, phone, firstName) {
    return dispatch => {
        dispatch(request({ username }));

        userService.register(username, password, phone, firstName)
            .then(
                user => {
                    dispatch(success(user));
                    dispatch(alertActions.success(responseMessages.REGISTER_SUCCESS, 15000));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request(user) { return { type: userConstants.REGISTER_REQUEST, user } }
    function success(user) { return { type: userConstants.REGISTER_SUCCESS, user } }
    function failure(error) { return { type: userConstants.REGISTER_FAILURE, error } }

}

function logout() {
    userService.logout();
    return { type: userConstants.LOGOUT };
}

function updateProfile(profile) {
    return dispatch => {
        dispatch(request());

        userService.updateProfile(profile)
            .then(
                response => {
                    updateAuthEmail(response.data.email);
                    dispatch(success(response.data));
                    localStorage.setItem('profile', JSON.stringify(response.data));
                    dispatch(alertActions.success(responseMessages.PROFILE_UPDATED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request() { return { type: userConstants.UPDATE_PROFILE_REQUEST } }
    function success(profile) { return { type: userConstants.UPDATE_PROFILE_SUCCESS, profile } }
    function failure(error) { return { type: userConstants.UPDATE_PROFILE_FAILURE, error } }
}

function updateCompanyInfo(companyInfo) {
    return dispatch => {
        dispatch(request());

        userService.updateCompanyInfo(companyInfo)
            .then(
                response => {
                    dispatch(success(response.data));
                    dispatch(alertActions.success(responseMessages.COMPANY_INFO_UPDATED_SUCCESSFULLY));
                    localStorage.setItem('companyInfo', JSON.stringify(response.data));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request() { return { type: userConstants.UPDATE_COMPANY_INFO_REQUEST } }
    function success(companyInfo) { return { type: userConstants.UPDATE_COMPANY_INFO_SUCCESS, companyInfo } }
    function failure(error) { return { type: userConstants.UPDATE_COMPANY_INFO_FAILURE, error } }
}

function getCompanyInfo() {
    return dispatch => {
        dispatch(request());

        userService.getCompanyInfo()
            .then(
                companyInfo => {
                    dispatch(success(companyInfo.data));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request() { return { type: userConstants.GET_COMPANY_INFO_REQUEST } }
    function success(companyInfo) { return { type: userConstants.GET_COMPANY_INFO_SUCCESS, companyInfo } }
    function failure(error) { return { type: userConstants.GET_COMPANY_INFO_FAILURE, error } }
}

function updateAuthEmail(newEmail) {
    let user = JSON.parse(localStorage.getItem('user'));

    if (user.username !== newEmail) {
        user.username = newEmail;

        let currentAuth = new Buffer(user.auth.substring(6), 'base64').toString();

        user.auth = 'Basic ' + new Buffer(newEmail + ":" + currentAuth.split(':')[1]).toString('base64');

        localStorage.setItem('user', JSON.stringify(user));
    }
}

function getRelatedUsers() {
    return dispatch => {
        dispatch(request());

        userService.getRelatedUsers()
            .then(
                users => {
                    dispatch(success(users));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request() { return { type: userConstants.GET_RELATED_USERS_REQUEST } }
    function success(users) { return { type: userConstants.GET_RELATED_USERS_SUCCESS, users } }
    function failure(error) { return { type: userConstants.GET_RELATED_USERS_FAILURE, error } }
}

function createRelatedUser(user) {
    return dispatch => {
        dispatch(request());

        userService.createRelatedUser(user)
            .then(
                response => {
                    dispatch(success(response.data));
                    dispatch(alertActions.success(responseMessages.USER_CREATED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request() { return { type: userConstants.CREATE_RELATED_USER_REQUEST } }
    function success(user) { return { type: userConstants.CREATE_RELATED_USER_SUCCESS, user } }
    function failure(error) { return { type: userConstants.CREATE_RELATED_USER_FAILURE, error } }
}

function deleteRelatedUser(userId, email) {
    return dispatch => {
        dispatch(request());

        userService.deleteRelatedUser(userId)
            .then(
                user => {
                    dispatch(success(email));
                    dispatch(alertActions.success(responseMessages.USER_DELETED_SUCCESSFULLY));
                },
                error => {
                    dispatch(failure(error.toString()));
                    dispatch(alertActions.error(error.toString()));
                }
            );
    };

    function request() { return { type: userConstants.DELETE_RELATED_USER_REQUEST } }
    function success(email) { return { type: userConstants.DELETE_RELATED_USER_SUCCESS, email } }
    function failure(error) { return { type: userConstants.DELETE_RELATED_USER_FAILURE, error } }
}
