import React from "react";
import logoTestLab from "../images/logo_test_lab.svg";
import {Icon, Nav, Sidenav} from "rsuite";
import {history} from "../helpers/history";
import {connect} from "react-redux";
import testsIcon from "../images/tests-icon.svg";
import questionIcon from "../images/question.svg";
import barChartIcon from "../images/bar-chart.svg";
import interviewerIcon1 from "../images/interviewer_1.svg"
import interviewerIcon2 from "../images/interviewer_2.svg"
import interviewerIcon3 from "../images/interviewer_3.svg"
import {pathConstants} from "../constants/path.constants";
import {language} from "../content/language";
import {hasAuthority} from "../helpers/auth";
import {authoritiesConstants} from "../constants/authorities.constants";
import loyaltyLog from "../helpers/loyaltyLog";
import {store} from "../helpers/store";
import {alertActions} from "../actions/alert.actions";
import {responseMessages} from "../content/responseMessages";
import {userService} from "../services/user.service";

class SideNav extends React.Component {

    constructor(props) {
        super(props);
        this.state = {
            isSubscriptionActive: '',
        }
    }

    openPathIfSubscriptionActive(path) {
        if (this.state.isSubscriptionActive === true) {
            history.push(path);
        } else {
            store.dispatch(alertActions.error(responseMessages.SUBSCRIPTION_IS_OVER));
            history.push(pathConstants.DEPOSIT);
        }
    }

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT SIDENAV");

        let localItem = localStorage.getItem('isSubscriptionActive');

        if (localItem == null) {
            userService.getBalance()
                .then(
                    response => {
                        localItem = response.data.subscriptionUntil > Date.now();
                        localStorage.setItem('isSubscriptionActive', localItem);
                        this.setState({isSubscriptionActive: localItem});
                    }
                );
        } else { this.setState({isSubscriptionActive: localItem==='true'});}
    };

    render() {

        loyaltyLog("render component sidenav");

        const hasBalance = hasAuthority(authoritiesConstants.ROLE_ADMIN);
        const hasTestViewPermit = hasAuthority(authoritiesConstants.ROLE_USER);
        const hasCandidateTestViewPermit = hasAuthority(authoritiesConstants.ROLE_USER);
        const hasSectionViewPermit = hasAuthority(authoritiesConstants.ROLE_EDITOR);
        return (
            <div>
                <Sidenav>
                    <Sidenav.Body>
                        <svg viewBox={logoTestLab.viewBox} style={{margin: '1% 12% 5% 5%', cursor: 'pointer'}}
                             onClick={() => {
                                 this.openPathIfSubscriptionActive(pathConstants.TESTS_LIST)
                             }}>
                            <use xlinkHref={`#${logoTestLab.id}`}/>
                        </svg>
                        <Nav>
                            <Nav.Item eventKey="1" icon={<Icon icon={testsIcon}/>}
                                      onClick={() => {
                                          this.openPathIfSubscriptionActive(pathConstants.TESTS_LIST)
                                      }}
                                      style={{display: hasTestViewPermit ? 'inherit' : 'none'}}
                                      active={this.props.parentProps.location.pathname.includes(pathConstants.TESTS_LIST) || "/" === this.props.parentProps.location.pathname}>
                                {language.TESTS}
                            </Nav.Item>

                            <Nav.Item eventKey="2" icon={<Icon icon={questionIcon}/>}
                                      onClick={() => {
                                          history.push(pathConstants.TOPICS_LIST)
                                      }}
                                      style={{display: hasSectionViewPermit ? 'inherit' : 'none'}}
                                      active={this.props.parentProps.location.pathname.includes(pathConstants.TOPICS_LIST)}>
                                {language.TOPICS_QUESTIONS}
                            </Nav.Item>

                            <Nav.Item eventKey="3" icon={<Icon icon={barChartIcon}/>}
                                      onClick={() => {
                                          this.openPathIfSubscriptionActive(pathConstants.CANDIDATE_TESTS_LIST)
                                      }}
                                      style={{display: hasCandidateTestViewPermit ? 'inherit' : 'none', fill: 'red'}}
                                      active={this.props.parentProps.location.pathname.includes(pathConstants.CANDIDATE_TESTS_LIST)}>
                                {language.TEST_RESULTS}
                            </Nav.Item>
                        </Nav>
                        <InterviewerIconElement path={this.props.parentProps.location.pathname}/>
                    </Sidenav.Body>
                </Sidenav>
            </div>
        );
    }

}
const InterviewerIconElement = ({ path, ...props }) => {

    if (!path || path.includes('candidate-tests')) {
        return (
            <svg viewBox={interviewerIcon1.viewBox} style={{margin: '40% 15% 0 12%'}}>
                <use xlinkHref={`#${interviewerIcon1.id}`} />
            </svg>
        );
    }
    if (path.includes('settings') || path.includes('faq') || path.includes('deposit')
            || path.includes('bills')) {
        return (
            <svg viewBox={interviewerIcon2.viewBox} style={{margin: '40% 15% 0 12%'}}>
                <use xlinkHref={`#${interviewerIcon2.id}`} />
            </svg>
        );
    }
    if (path.includes('tests') || path === "/") {
        return (
            <svg viewBox={interviewerIcon3.viewBox} style={{margin: '40% 15% 0 12%'}}>
                <use xlinkHref={`#${interviewerIcon3.id}`} />
            </svg>
        );
    }
    return null;
};

function mapStateToProps(state) {
    const {} = state;
    return {
    };
}

const connectedSideNav = connect(mapStateToProps)(SideNav);
export { connectedSideNav as SideNav };