import React from "react";
import {connect} from "react-redux";
import {
    ButtonToolbar,
    Col,
    ControlLabel,
    Form,
    FormControl,
    FormGroup,
    Grid,
    Icon,
    Row,
    Schema,
    SelectPicker
} from "rsuite";
import {language} from "../../content/language";
import {alertActions} from "../../actions/alert.actions";
import {userActions} from "../../actions/user.actions";
import {nullToEmptyStr, nullToZero} from "../../helpers/util";
import saveButton from "../../images/save_button.svg";
import {SaveButton} from "../buttons/SaveButton";
import {focusError} from "../../helpers/form.helper";
import loyaltyLog from "../../helpers/loyaltyLog";
import {userService} from "../../services/user.service";


class CompanyInfo extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT CompanyInfo");

        userService.getCompanyInfo().then(companyInfo => {
            this.setState({
                formValue: {
                    id: nullToZero(companyInfo.data.id),
                    typeComp: nullToEmptyStr(companyInfo.data.typeComp),
                    legalNameComp: nullToEmptyStr(companyInfo.data.legalNameComp),
                    inn: nullToEmptyStr(companyInfo.data.inn),
                    kpp: nullToEmptyStr(companyInfo.data.kpp),
                    ogrn: nullToEmptyStr(companyInfo.data.ogrn),
                    bik: nullToEmptyStr(companyInfo.data.bik),
                    paymentAccount: nullToEmptyStr(companyInfo.data.paymentAccount),
                    correspondentAccount: nullToEmptyStr(companyInfo.data.correspondentAccount),
                    bankName: nullToEmptyStr(companyInfo.data.bankName),
                    legalAddress: nullToEmptyStr(companyInfo.data.legalAddress),
                    actualAddress: nullToEmptyStr(companyInfo.data.actualAddress),
                    nameCeo: nullToEmptyStr(companyInfo.data.nameCeo),
                    namePost: nullToEmptyStr(companyInfo.data.namePost),
                    postAddress: nullToEmptyStr(companyInfo.data.postAddress)
                }
            });
        }, error => {
            this.props.dispatch(alertActions.error(language.CANNOT_LOAD_COMPANY_INFO));
        });
        this.setState({
            companyTypes: [
                {
                    value: 'IP',
                    label: 'ИП'
                }, {
                    value: 'OOO',
                    label: 'ООО'
                }, {
                    value: 'AO',
                    label: 'АО'
                }, {
                    value: 'PAO',
                    label: 'ПАО'
                }, {
                    value: 'ZAO',
                    label: 'ЗАО'
                },
            ]
        });
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT CompanyInfo");
        super(props);

        this.state = {
            formValue: {
                id: '',
                typeComp: '',
                legalNameComp: '',
                inn: '',
                kpp: '',
                ogrn: '',
                bik: '',
                paymentAccount: '',
                correspondentAccount: '',
                bankName: '',
                legalAddress: '',
                actualAddress: '',
                nameCeo: '',
                namePost: '',
                postAddress: ''
            },
            formError: {},
            companyTypes: []
        };
    }

    updateCompanyInfo = () => {
        const {formValue} = this.state;
        if (!this.formProfile.check()) {
            return;
        }
        this.props.dispatch(userActions.updateCompanyInfo(formValue));
    };

    render() {
        loyaltyLog("RENDER COMPONENT CompanyInfo");

        const {} = this.props;
        const {formValue, formError, companyTypes} = this.state;

        return (
            <Grid fluid>
                <Row>
                    <Col xs={24} sm={24} md={24} lg={13} style={{padding: 0}}>
                        <Form className={"ll-form-page"} layout="horizontal"
                              onChange={formValue => {
                                  this.setState({formValue});
                              }} formValue={formValue}
                              ref={ref => (this.formProfile = ref)}
                              onCheck={formError => this.setState({formError})} formError={formError}
                              model={companyInfoModel}
                              onError={errors => focusError(errors)}>

                            <FormGroup className={"ll-form-header"}>
                                <h3>{language.INFO}</h3>
                            </FormGroup>

                            <FormGroup style={{fontWeight: 600}}>
                                {language.COMPANY_INFO}
                            </FormGroup>

                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.TYPE_COMP}</ControlLabel>
                                <FormControl accepter={SelectPicker} placeholder={language.TYPE_COMP_PLACEHOLDER}
                                             name="typeComp"
                                             data={companyTypes} searchable={false} style={{width: 350}}
                                             cleanable={false}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.LEGAL_NAME_COMP}</ControlLabel>
                                <FormControl name="legalNameComp" style={{width: 350}}
                                             placeholder={language.LEGAL_NAME_COMP_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.INN}</ControlLabel>
                                <FormControl name="inn" style={{width: 350}} placeholder={language.INN_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.KPP}</ControlLabel>
                                <FormControl name="kpp" style={{width: 350}} placeholder={language.KPP_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.OGRN}</ControlLabel>
                                <FormControl name="ogrn" style={{width: 350}} placeholder={language.OGRN_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.BIK}</ControlLabel>
                                <FormControl name="bik" style={{width: 350}} placeholder={language.BIK_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.PAYMENT_ACCOUNT}</ControlLabel>
                                <FormControl name="paymentAccount" style={{width: 350}}
                                             placeholder={language.PAYMENT_ACCOUNT_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.CORRESPONDENT_ACCOUNT}</ControlLabel>
                                <FormControl name="correspondentAccount" style={{width: 350}}
                                             placeholder={language.CORRESPONDENT_ACCOUNT_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.BANK_NAME}</ControlLabel>
                                <FormControl name="bankName" style={{width: 350}}
                                             placeholder={language.BANK_NAME_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.LEGAL_ADDRESS}</ControlLabel>
                                <FormControl name="legalAddress" style={{width: 350}}
                                             placeholder={language.LEGAL_ADDRESS_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.ACTUAL_ADDRESS}</ControlLabel>
                                <FormControl name="actualAddress" style={{width: 350}}
                                             placeholder={language.ACTUAL_ADDRESS_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.NAME_CEO}</ControlLabel>
                                <FormControl name="nameCeo" style={{width: 350}}
                                             placeholder={language.NAME_CEO_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.NAME_POST}</ControlLabel>
                                <FormControl name="namePost" style={{width: 350}}
                                             placeholder={language.NAME_POST_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.POST_ADDRESS}</ControlLabel>
                                <FormControl name="postAddress" style={{width: 350}}
                                             placeholder={language.POST_ADDRESS_PLACEHOLDER}/>
                            </FormGroup>

                            <FormGroup>
                                <ButtonToolbar style={{marginTop: 15}}>
                                    <SaveButton hasEditPermit={true} icon={<Icon icon={saveButton}/>}
                                                onClickAction={this.updateCompanyInfo}/>
                                </ButtonToolbar>
                            </FormGroup>
                        </Form>
                    </Col>
                </Row>
            </Grid>
        );
    }
}

const {NumberType} = Schema.Types;
const {StringType} = Schema.Types;
const companyInfoModel = Schema.Model({
    id: NumberType(language.INSERT_NUMBER),
    userId: NumberType(language.INSERT_NUMBER),
    typeComp: StringType().isRequired(language.REQUIRED_INPUT).maxLength(64, language.STRING_MAX_LENGTH(64)),
    legalNameComp: StringType().isRequired(language.REQUIRED_INPUT).maxLength(64, language.STRING_MAX_LENGTH(64)),
    inn: StringType().isRequired(language.REQUIRED_INPUT).containsNumber(language.INSERT_NUMBER).minLength(10, language.STRING_MIN_LENGTH(10)).maxLength(10, language.STRING_MAX_LENGTH(12)),
    kpp: StringType().isRequired(language.REQUIRED_INPUT).containsNumber(language.INSERT_NUMBER).minLength(9, language.STRING_MIN_LENGTH(9)).maxLength(9, language.STRING_MAX_LENGTH(9)),
    ogrn: StringType().containsNumber(language.INSERT_NUMBER).minLength(13, language.STRING_MIN_LENGTH(13)).maxLength(15, language.STRING_MAX_LENGTH(15)),
    bik: StringType().containsNumber(language.INSERT_NUMBER).minLength(9, language.STRING_MIN_LENGTH(9)).maxLength(9, language.STRING_MAX_LENGTH(9)),
    paymentAccount: StringType().containsNumber(language.INSERT_NUMBER).minLength(20, language.STRING_MIN_LENGTH(20)).maxLength(20, language.STRING_MAX_LENGTH(20)),
    correspondentAccount: StringType().containsNumber(language.INSERT_NUMBER).minLength(20, language.STRING_MIN_LENGTH(20)).maxLength(20, language.STRING_MAX_LENGTH(20)),
    bankName: StringType().maxLength(64, language.STRING_MAX_LENGTH(64)),
    legalAddress: StringType().maxLength(64, language.STRING_MAX_LENGTH(256)),
    actualAddress: StringType().maxLength(64, language.STRING_MAX_LENGTH(256)),
    nameCeo: StringType().maxLength(64, language.STRING_MAX_LENGTH(64)),
    namePost: StringType().maxLength(64, language.STRING_MAX_LENGTH(64)),
    postAddress: StringType().maxLength(64, language.STRING_MAX_LENGTH(256)),

});

function mapStateToProps(state) {
    const {} = state;
    return {};
}

const connectedCompanyInfo = connect(mapStateToProps)(CompanyInfo);
export {connectedCompanyInfo as CompanyInfo};