import React, {Component} from "react";
import {connect} from "react-redux";
import {language} from "../../content/language";
import {alertActions} from "../../actions/alert.actions";
import {nullToEmptyStr} from "../../helpers/util";
import {Button, Col, Divider, Form, Grid, Icon, IconButton, Modal, Row} from "rsuite";
import basicPriceLogo from "../../images/basic_price_logo.svg"
import proPriceLogo from "../../images/pro_price_logo.svg"
import premiumPriceLogo from "../../images/premium_price_logo.svg"
import detailedDepositButton from "../../images/detailed_deposit_button.svg"
import loyaltyLog from "../../helpers/loyaltyLog";
import {orderActions} from "../../actions/order.actions";


class DepositScreen extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT DEPOSIT SCREEN");

        this.jivo();

        const prices = {
            basicPrice: 9990,
            proPrice: 22500,
            premiumPrice: 90000,
            basicTests: 50,
            proTests: 200,
            premiumTests: 1000
        };

        if (prices) {
            this.setState({
                basicPrice: nullToEmptyStr(prices.basicPrice),
                proPrice: nullToEmptyStr(prices.proPrice),
                premiumPrice: nullToEmptyStr(prices.premiumPrice),
                basicTests: nullToEmptyStr(prices.basicTests),
                proTests: nullToEmptyStr(prices.proTests),
                premiumTests: nullToEmptyStr(prices.premiumTests)
            });
        } else {
            this.props.dispatch(alertActions.error(language.CANNOT_LOAD_PRICES));
        }
    }

    jivo() {
        const script = document.createElement("script");

        script.src = "https://code.jivo.ru/widget/HAeYKh2mO7";
        script.async = true;
        script.className = "jivo"

        document.body.appendChild(script);
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT USER SETTINGS");
        super(props);

        this.state = {
            basicPrice: '',
            proPrice: '',
            premiumPrice: '',
            basicTests: '',
            proTests: '',
            premiumTests: '',
            showPaymentTypeModal: false,
            rateId: '',
            rateName: '',
            rateAmount: '',
            jivo: ''
        };
    }

    render() {

        loyaltyLog("RENDER COMPONENT DEPOSIT SCREEN");

        const { basicPrice, proPrice, premiumPrice, basicTests, proTests, premiumTests, showPaymentTypeModal,
                    rateId, rateName, rateAmount} = this.state;

        return (
            <Grid fluid style={{paddingTop: 46, paddingBottom: 150}}>
                <Row>
                    <Col>
                        <h3>{language.ACCOUNT_DEPOSIT}</h3>
                    </Col>
                </Row>
                <Divider />
                <Row style={{paddingTop: 15, textAlign: 'center'}}>
                    <Col sm={24} md={8}>
                        <h4>{language.BASIC_PRICE}</h4>
                        <h4 style={{marginTop: 10, marginBottom: 55}}>{basicTests}{language.TEST_NUMBER_SUFFIX}</h4>
                        <Icon icon={basicPriceLogo} className={"basic-price-logo"}/>
                        <p>{basicPrice}{language.RUB}</p>
                        <h6 style={{marginTop: 26, marginBottom: 22}}>{Math.floor(basicPrice/basicTests)}{language.PER_TEST}</h6>
                        <DepositButton setChanged={() => {
                            this.setState({
                                showPaymentTypeModal: true,
                                rateName: language.BASIC_PRICE,
                                rateId: 3,
                                rateAmount: basicPrice
                            });
                        }}/>
                    </Col>
                    <Col sm={24} md={8}>
                        <h4>{language.PRO_PRICE}</h4>
                        <h4 style={{marginTop: 10, marginBottom: 45}}>{proTests}{language.TEST_NUMBER_SUFFIX}</h4>
                        <Icon icon={proPriceLogo} className={"pro-price-logo"}/>
                        <p>{proPrice}{language.RUB}</p>
                        <h6 style={{marginTop: 5}}>{Math.floor(proPrice/proTests)}{language.PER_TEST}</h6>
                        <h6 style={{marginTop: 10}}>{language.PACK_BENEFIT}{Math.floor((basicPrice/basicTests  - proPrice/proTests) * proTests)}</h6>
                        <DepositButton setChanged={() => {
                            this.setState({
                            showPaymentTypeModal: true,
                            rateName: language.PRO_PRICE,
                            rateId: 4,
                            rateAmount: proPrice
                        });
                        }}/>
                    </Col>
                    <Col sm={24} md={8}>
                        <h4>{language.PREMIUM_PRICE}</h4>
                        <h4 style={{marginTop: 10, marginBottom: 50}}>{premiumTests}{language.TEST_NUMBER_SUFFIX}</h4>
                        <Icon icon={premiumPriceLogo} className={"premium-price-logo"}/>
                        <p>{premiumPrice}{language.RUB}</p>
                        <h6 style={{marginTop: 5}}>{Math.floor(premiumPrice/premiumTests)}{language.PER_TEST}</h6>
                        <h6 style={{marginTop: 10}}>{language.PACK_BENEFIT}{Math.floor((basicPrice/basicTests - premiumPrice/premiumTests) * premiumTests)}</h6>
                        <DepositButton setChanged={() => {
                            this.setState({
                                showPaymentTypeModal: true,
                                rateName: language.PRO_PRICE,
                                rateId: 5,
                                rateAmount: premiumPrice
                            });
                        }}/>
                    </Col>
                </Row>
                <PaymentTypeModal showPaymentTypeModal={showPaymentTypeModal} rateId={rateId}
                                  rateName={rateName} rateAmount={rateAmount}
                                       closeModal={() => {
                                           this.setState({showPaymentTypeModal: false})
                                       }}
                                       dispatch={this.props.dispatch}/>
            </Grid>
        );
    }
}

class PaymentTypeModal extends Component {

    constructor(props) {
        super(props);
        this.state = {
            show: false,
            modalFormValue: {
                rateName: '',
                rateId: '',
                rateAmount: ''
            },
            modalFormError: {}
        }
    }

    UNSAFE_componentWillReceiveProps(props) {
        this.setState({
            show: props.showPaymentTypeModal,
            closeModal: props.closeModal,
            modalFormValue: {
                rateName: props.rateName || '',
                rateId: props.rateId || '',
                rateAmount: props.rateAmount || ''
            }
        })
    }

    render() {
        loyaltyLog("RENDER PAYMENT TYPE MODAL");

        const { modalFormValue, modalFormError, show, closeModal } = this.state;

        return (<Modal show={this.state.show} onHide={() => this.setState({show: false})}>
            <Modal.Header>
                <Modal.Title>{language.PAYMENT_TYPE_TITLE}</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <p style={{textAlign: 'center'}}>{language.PAYMENT_TYPE_RATE_NAME(modalFormValue.rateName)}</p>
                <p style={{textAlign: 'center'}}>{language.PAYMENT_TYPE_RATE_PRICE(modalFormValue.rateAmount)}</p>
                <Form style={{padding: 15}}
                      formValue={modalFormValue}
                      formError={modalFormError}
                      onChange={modalFormValue => {
                          this.setState({modalFormValue : modalFormValue});
                      }}
                      ref={ref => (this.form = ref)}
                      checkTrigger={'blur'}>

                    <Button onClick={() => {
                        this.props.dispatch(orderActions.createOrder({
                            rateId: modalFormValue.rateId,
                            paymentMethod: 'WIRE_TRANSFER'
                        }));
                        this.setState({show: false, modalFormValue: { rateName: '', rateId:'', rateAmount:''}});
                        closeModal();
                    }} appearance="primary" style={{textAlign: 'center', width: '100%'}}>
                        {language.PAYMENT_TYPE_INVOICE}
                    </Button>
                </Form>
            </Modal.Body>
            <Modal.Footer>
                <Button onClick={() => { this.setState({show: false, modalFormValue: { rateName: '', rateId:'', rateAmount:''}}); closeModal();}} appearance="subtle">
                    {language.CANCEL}
                </Button>
            </Modal.Footer>
        </Modal>)
    }
}

const DepositButton = ({setChanged}) => {
    return (
        <IconButton className={"deposit-button-amount"}
                    style={{marginTop: 70}}
                    appearance="subtle"
                    onClick={() => setChanged()}
                    icon={<Icon icon={detailedDepositButton}/>}
        />
    );
};

function mapStateToProps(state) {
    const { } = state;
    return {
    };
}

const connectedDepositScreen = connect(mapStateToProps)(DepositScreen);
export { connectedDepositScreen as DepositScreen };