import React from "react";
import {connect} from "react-redux";
import {
    ButtonToolbar,
    Col,
    ControlLabel,
    Form,
    FormControl,
    FormGroup,
    Grid,
    Icon, IconButton, Nav,
    Row,
    Schema
} from "rsuite";
import {language} from "../../content/language";
import {getStoragedProfile} from "../../helpers/profile.helper";
import {alertActions} from "../../actions/alert.actions";
import {userActions} from "../../actions/user.actions";
import {nullToEmptyStr} from "../../helpers/util";
import {utilConstants} from "../../constants/util.constants";
import saveButton from "../../images/save_button.svg";
import {SaveButton} from "../buttons/SaveButton";
import {hasAuthority} from "../../helpers/auth";
import {authoritiesConstants} from "../../constants/authorities.constants";
import {responseMessages} from "../../content/responseMessages";
import createUserIcon from "../../images/connect_button.svg";
import deleteUserIcon from "../../images/trash_icon.svg";
import {focusError} from "../../helpers/form.helper";
import loyaltyLog from "../../helpers/loyaltyLog";
import {history} from "../../helpers/history";
import {pathConstants} from "../../constants/path.constants";


class UserSettings extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT USER SETTINGS");

        const canAddUsers = hasAuthority(authoritiesConstants.ROLE_ADMIN);
        const profile = getStoragedProfile();
        if (profile) {
            this.setState({
                formValue: {
                    id: nullToEmptyStr(profile.id),
                    email: nullToEmptyStr(profile.email),
                    firstName: nullToEmptyStr(profile.firstName),
                    phone: nullToEmptyStr(profile.phone)
                },
                formValuePassword: {
                    id: nullToEmptyStr(profile.id),
                    email: nullToEmptyStr(profile.email),
                    firstName: nullToEmptyStr(profile.firstName),
                    phone: nullToEmptyStr(profile.phone),
                    password: '',
                    verifyPassword: ''
                },
                canAddUsers: canAddUsers
            });
        } else {
            this.props.dispatch(alertActions.error(language.CANNOT_LOAD_PROFILE));
        }
        if (canAddUsers) {
            this.props.dispatch(userActions.getRelatedUsers());
        }
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT USER SETTINGS");
        super(props);

        this.state = {
            formValue: {
                id: '',
                email: '',
                firstName: '',
                phone: ''
            },
            formError: {},
            canAddUsers: false,
            newUserFormValue: {
                email: ''
            },
            newUserFormError: {},
            formValuePassword: {
                id: '',
                email: '',
                firstName: '',
                phone: '',
                password: '',
                verifyPassword: ''
            },
            formValuePasswordError: {},
        };
    }

    updateProfile = () => {
        const {formValue} = this.state;
        if (!this.formProfile.check()) {
            return;
        }
        this.props.dispatch(userActions.updateProfile(formValue));
    };

    updatePassword = () => {
        const {formValuePassword} = this.state;
        if (!this.formPassword.check()) {
            return;
        }
        this.props.dispatch(userActions.updateProfile(formValuePassword));
    };

    render() {

        loyaltyLog("RENDER COMPONENT USER SETTINGS");

        const {relatedUsersList} = this.props;
        const {
            formValue,
            formError,
            newUserFormValue,
            newUserFormError,
            canAddUsers,
            formValuePassword,
            formValuePasswordError
        } = this.state;

        return (
            <Grid fluid>

                <Row>
                    <Col xs={24} sm={24} md={24} lg={13} style={{padding: 0}}>
                        <Form className={"ll-form-page"} layout="horizontal"
                              onChange={formValue => {
                                  this.setState({formValue});
                              }} formValue={formValue}
                              ref={ref => (this.formProfile = ref)}
                              onCheck={formError => this.setState({formError})} formError={formError}
                              model={profileModel}
                              onError={errors => focusError(errors)}>

                            <FormGroup className={"ll-form-header"}>
                                <h3>{language.SETTINGS}</h3>
                            </FormGroup>

                            <FormGroup style={{fontWeight: 600}}>
                                {language.PROFILE}
                            </FormGroup>

                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.EMAIL}</ControlLabel>
                                <FormControl name="email" style={{width: 350}}
                                             placeholder={language.EMAIL_PLACEHOLDER}/>
                            </FormGroup>
                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.PERSON_NAME}</ControlLabel>
                                <FormControl name="firstName" style={{width: 350}}
                                             placeholder={language.NAME_PLACEHOLDER}/>
                            </FormGroup>

                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.PHONE}</ControlLabel>
                                <FormControl name="phone" style={{width: 350}}
                                             placeholder={language.PHONE_PLACEHOLDER}/>
                            </FormGroup>

                            <FormGroup>
                                <ButtonToolbar style={{marginTop: 15}}>
                                    <SaveButton hasEditPermit={true} icon={<Icon icon={saveButton}/>}
                                                onClickAction={this.updateProfile}/>
                                </ButtonToolbar>
                            </FormGroup>
                        </Form>
                    </Col>

                    <Col xs={24} sm={24} md={24} lg={11} style={{padding: 0}}>
                        <SubUsersView users={relatedUsersList} newUserFormValue={newUserFormValue}
                                      newUserFormError={newUserFormError} parent={this} hasRelatedUsers={canAddUsers}/>
                        <FormGroup style={{cursor: 'pointer', fontWeight: 600}} onClick={() => { history.push(pathConstants.COMPANY_INFO)}} >
                            <a>{language.COMPANY_INFO}</a>
                        </FormGroup>
                    </Col>
                </Row>

                <Row>
                    <Col xs={24} sm={24} md={24} lg={13} style={{padding: 0}}>
                        <Form layout="horizontal" style={{paddingTop: 0, paddingBottom: 46}}
                              onChange={formValue => {
                                  this.setState(
                                      {
                                          formValuePassword: {
                                              "id": this.state.formValue.id,
                                              "email": this.state.formValue.email,
                                              "firstName": this.state.formValue.firstName,
                                              "phone": this.state.formValue.phone,
                                              "password": formValue.password,
                                              "verifyPassword": formValue.verifyPassword
                                          }
                                      });
                              }}
                              formValue={formValuePassword}
                              ref={ref => (this.formPassword = ref)}
                              onCheck={formValuePasswordError => this.setState({formValuePasswordError})}
                              formError={formValuePasswordError} model={passwordModel}
                              onError={errors => focusError(errors)}>

                            <FormGroup style={{fontWeight: 600}}>
                                {language.PASSWORD_CHANGE}
                            </FormGroup>

                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.PASSWORD_INSERT}</ControlLabel>
                                <FormControl name="password" style={{width: 350}} type="password"/>
                            </FormGroup>

                            <FormGroup className={"ll-settings-form-group"}>
                                <ControlLabel>{language.PASSWORD_INSERT_REPEAT}</ControlLabel>
                                <FormControl name="verifyPassword" style={{width: 350}} type="password"/>
                            </FormGroup>

                            <FormGroup>
                                <ButtonToolbar style={{marginTop: 15}}>
                                    <SaveButton hasEditPermit={true} icon={<Icon icon={saveButton}/>}
                                                onClickAction={this.updatePassword}/>
                                </ButtonToolbar>
                            </FormGroup>
                        </Form>
                    </Col>
                    <Col xs={24} sm={24} md={24} lg={11} style={{padding: 0}}>
                        <Form layout="horizontal" style={{paddingTop: 0, paddingBottom: 46}}>

                        </Form>
                    </Col>
                </Row>
            </Grid>
        );
    }
}

const {NumberType} = Schema.Types;
const {StringType} = Schema.Types;
const profileModel = Schema.Model({
    id: NumberType(language.INSERT_NUMBER),
    firstName: StringType(),
    email: StringType().isEmail(language.EMAIL_VALIDATOR)
        .isRequired(language.REQUIRED_INPUT),
    phone: NumberType(language.PHONE_VALIDATOR).range(utilConstants.PHONE_NUMBER_MIN, utilConstants.PHONE_NUMBER_MAX, language.PHONE_VALIDATOR)
});
const newUserModel = Schema.Model({
    email: StringType().isEmail(language.EMAIL_VALIDATOR)
        .isRequired(language.REQUIRED_INPUT)
});
const passwordModel = Schema.Model({
    password: StringType().isRequired(language.REQUIRED_INPUT),
    verifyPassword: StringType()
        .addRule((value, data) => {
            return value === data.password;
        }, language.PASSWORD_MATCH)
        .isRequired(language.REQUIRED_INPUT)
});

const SubUsersView = ({users, newUserFormValue, newUserFormError, parent, hasRelatedUsers, ...props}) => {
    if (!hasRelatedUsers) return null;
    return (
        <>
            <Form className={"ll-form-page"} layout="horizontal"
                  onChange={newUserFormValue => {
                      parent.setState({newUserFormValue});
                  }} formValue={newUserFormValue}
                  ref={ref => (parent.newUserForm = ref)}
                  onCheck={newUserFormError => parent.setState({newUserFormError})} formError={newUserFormError}
                  model={newUserModel}>

                <FormGroup className={"ll-form-header"} style={{marginTop: 72}}>
                </FormGroup>

                <FormGroup style={{fontWeight: 600}}>
                    {language.RELATED_USERS}
                </FormGroup>
                {
                    users.items.map((user, index) => {
                        return (
                            <FormGroup key={index} className={"ll-settings-form-group"}>
                                <ControlLabel style={{width: 70}}>{language.EMAIL}</ControlLabel>
                                <p className={"related-user-email-p"}>
                                    {user.email}</p>
                                <DeleteUserButton parent={parent} userId={user.id} email={user.email}/>
                            </FormGroup>
                        )
                    })
                }
                <FormGroup className={"ll-settings-form-group"}>
                    <ControlLabel style={{width: 90}}>{language.EMAIL}</ControlLabel>
                    <FormControl name="email" style={{width: 380}} placeholder={language.NEW_OWN_USER_PLACEHOLDER}/>
                </FormGroup>

                <FormGroup>
                    <CreateUserButton parent={parent} newUserModel={newUserFormValue}/>
                </FormGroup>
            </Form>
        </>
    )
};

const CreateUserButton = ({parent, newUserModel, ...props}) => {
    function handleAction() {
        if (parent.newUserForm.check()) {
            parent.props.dispatch(userActions.createRelatedUser(newUserModel));
        }
    }

    return (
        <IconButton
            style={{marginLeft: 102}}
            className={"create-related-user-logo"}
            appearance="subtle"
            onClick={handleAction}
            icon={<Icon icon={createUserIcon}/>}
        />
    );
};

const DeleteUserButton = ({parent, userId, email, ...props}) => {
    function handleAction() {
        if (confirm(responseMessages.USER_DELETE_CONFIRM)) {
            parent.props.dispatch(
                userActions.deleteRelatedUser(userId, email)
            );
        }
    }

    return (
        <IconButton
            appearance="subtle"
            onClick={handleAction}
            icon={<Icon icon={deleteUserIcon}/>}
        />
    );
};


function mapStateToProps(state) {
    const {relatedUsersList} = state;
    return {
        relatedUsersList
    };
}

const connectedUserSettings = connect(mapStateToProps)(UserSettings);
export {connectedUserSettings as UserSettings};