import React from "react";
import {connect} from "react-redux";
import {
    Checkbox, Col, Divider, FlexboxGrid, Loader, Panel, PanelGroup, Placeholder, Progress, Row, Tag, TagGroup
} from "rsuite";
import {testService} from "../../services/test.service";
import {BackButton} from "../buttons/BackButton";
import {language} from "../../content/language";
import {apiConstants} from "../../constants/api.constants";
import {styleConstants} from "../../constants/style.constants";
import {DownloadPDFButton} from "../buttons/DownloadPDFButton";
import {history} from "../../helpers/history";
import {pathConstants} from "../../constants/path.constants";
import {BackTextLink} from "../buttons/BackTextLink";
import loyaltyLog from "../../helpers/loyaltyLog";
const { Line } = Progress;


class CandidateTestDetails extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT CANDIDATE TEST DETAILS");

        let testId = this.props.match.params.id;

        testService.getCandidateTestById(testId).then(response => {
                if (response.status === apiConstants.STATUS_SUCCESS) {
                    this.setState({
                        candidateTest: response.data,
                        loading: false
                    });
                }
            }
        );
        testService.getCandidateQuestionsById(testId).then(response => {
                if (response.status === apiConstants.STATUS_SUCCESS) {
                    this.setState({
                        questions: response.data,
                    });
                }
            }
        );
    };

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT CANDIDATE TEST DETAILS");
        super(props);
        this.state = {
            candidateTest: {},
            questions: [],
            loading: true
        }
    };

    render() {
        loyaltyLog("RENDER COMPONENT CANDIDATE TEST DETAILS");

        const { candidateTest, questions, loading } = this.state;

        if (loading) {
            return (
                <div style={{height: 200}}>
                    <Loader backdrop content={language.LOADING} size="md" vertical center/>
                </div>
            );
        }
        return (
            <FlexboxGrid>
                <FlexboxGrid.Item colspan={24} className={"ll-table-header"}>
                <Row style={{paddingBottom: 30}}>
                    <Col xs={12}>
                        <h3>
                            {language.CANDIDATE_TEST_RESULT} {candidateTest.correctQuestionNumber} / {candidateTest.questionNumber}
                        </h3>
                    </Col>
                    <Col xs={12}>
                        <DownloadPDFButton onClickAction={() => alert("NOT READY YET")} hasEditPermit={true}
                                           className={"ll-test-result-download-pdf-button hidden"}/>
                    </Col>
                </Row>
                <BackTextLink action={() => history.push(pathConstants.CANDIDATE_TESTS_LIST)} text={language.BACK_TO_RESULTS_LIST}/>
                <Divider style={{marginTop: 10}}/>
                </FlexboxGrid.Item>
                <FlexboxGrid.Item colspan={12}>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.CANDIDATE_FIO}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {candidateTest.candidateName}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.TEST_TITLE}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {candidateTest.name}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.TEST_CREATED_DATE}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {candidateTest.created ? new Date(candidateTest.created).toLocaleString() : ''}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                </FlexboxGrid.Item>
                <FlexboxGrid.Item colspan={12}>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.TEST_STARTED_DATE}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {candidateTest.testStarted ? new Date(candidateTest.testStarted).toLocaleString() : '-'}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.TEST_FINISHED_DATE}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {candidateTest.testFinished ? new Date(candidateTest.testFinished).toLocaleString() : '-'}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.DURATION_MINUTES}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {language.DURATION_MINUTES_VALUE(Math.floor(candidateTest.duration / 60))}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                    <FlexboxGrid className={"ll-test-result-bio-line"}>
                        <FlexboxGrid.Item colspan={10}>
                            {language.FACT_DURATION_MINUTES}
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={14} style={{fontWeight: 'bold'}}>
                            {getTestLevelDescription(candidateTest.testStarted, candidateTest.testFinished, candidateTest.duration)}
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                </FlexboxGrid.Item>
                <FlexboxGrid.Item colspan={24} style={{marginTop: 55, paddingBottom: 50}}>
                    <Divider className={"ll-divider"}/>
                    <TopicStatistic topicStatistic={candidateTest.topicStatistic}/>
                    <Divider className={"ll-divider"}/>
                    <Questions questions={questions}/>
                    <BackTextLink action={() => history.push(pathConstants.CANDIDATE_TESTS_LIST)} text={language.BACK_TO_RESULTS_LIST}/>
                </FlexboxGrid.Item>
            </FlexboxGrid>
        )
    };
}

function getTestLevelDescription(testStarted, testFinished, duration) {
    if (!testStarted) return "-";
    if (!testFinished) testFinished = new Date();
    return language.FACT_DURATION_MINUTES_VALUE(Math.floor((( (new Date(testFinished) - new Date(testStarted)) % 86400000) % 3600000 ) / 60000),
        Math.floor(duration / 60))
}

function getDuration(resultPercent) {
    if (resultPercent < 16) return language.CANDIDATE_TEST_RESULT_OVERVIEW_BAD;
    if (resultPercent < 36) return language.CANDIDATE_TEST_RESULT_OVERVIEW_LESS_AVG;
    if (resultPercent < 51) return language.CANDIDATE_TEST_RESULT_OVERVIEW_GOOD;
    if (resultPercent < 80) return language.CANDIDATE_TEST_RESULT_OVERVIEW_ABOVE_AVG;
    return language.CANDIDATE_TEST_RESULT_OVERVIEW_PERFECT;
}

const Questions = ({questions, ...props}) => (
    <>
        <PanelGroup>
            <h4 style={{paddingTop: 20, paddingRight: 20, paddingLeft: 20, paddingBottom: 0}}>
                {language.CANDIDATE_TEST_DETAILS}
            </h4>
            {
                questions.map((question, index) => (
                    <Panel key={index} className={"ll-c-test-q-header"}>
                        <div style={{paddingBottom: 15, marginTop: 15}} dangerouslySetInnerHTML={{__html: question.description}}/>
                        <TagGroup >
                            <Tag style={{background: question.answeredCorrectly ? styleConstants.Q_CORRECT_BG : styleConstants.Q_WRONG_BG}}>
                                {question.answeredCorrectly ? language.CORRECT : language.INCORRECT}
                            </Tag>
                            <Tag>
                                {question.answered ? language.ANSWERED : language.NOT_ANSWERED}
                            </Tag>
                            <Tag>{question.topicName}</Tag>
                        </TagGroup>
                        <div style={{paddingTop: 15}}>
                        {
                            question.answers.map((answer, index) => (
                                <Checkbox style={{background: answer.correct ? styleConstants.Q_CORRECT_BG : answer.selected ? styleConstants.Q_WRONG_BG : 'white',
                                    borderRadius: 10, margin: '4px 0'}} disabled checked={answer.selected} key={index}>{answer.description}</Checkbox>
                            ))
                        }
                        </div>
                    </Panel>
                ))
            }
        </PanelGroup>
    </>
);

const TopicStatistic = ({topicStatistic, ...props}) => (
    <>
        <PanelGroup className={"ll-test-topic-stats"}>
        {
            topicStatistic.map((topic, index) => {
                const resultPercent = Math.floor(topic.correctQuestions / topic.questions * 100);
                return (
                    <FlexboxGrid key={index}>
                        <FlexboxGrid.Item colspan={12}>
                            <Panel key={index} header={topic.topic}>
                                <Line key={index} percent={resultPercent}
                                      status='active'/>
                            </Panel>
                        </FlexboxGrid.Item>
                        <FlexboxGrid.Item colspan={12}>
                            <Panel key={index} header={" "}>
                                {getTestLevelDescription(resultPercent)}
                            </Panel>
                        </FlexboxGrid.Item>
                    </FlexboxGrid>
                )
            })
        }
        </PanelGroup>
    </>
);

function mapStateToProps(state) {
    const {} = state;
    return {};
}

const connectedCandidateTestDetails = connect(mapStateToProps)(CandidateTestDetails);
export { connectedCandidateTestDetails as CandidateTestDetails };