import React, {Component} from "react"
import {connect} from "react-redux";
import {
    Button,
    ControlLabel,
    Form,
    FormControl,
    FormGroup,
    Icon,
    IconButton,
    Modal,
    Schema,
    Table,
    Tooltip,
    Whisper
} from "rsuite";
import {language} from "../../content/language";
import {styleConstants} from "../../constants/style.constants";
import loyaltyLog from "../../helpers/loyaltyLog";
import {orderActions} from "../../actions/order.actions";
import downloadButton from "../../images/download.svg";
import mailButton from "../../images/mail.svg";
import {StringType} from "schema-typed";
import {userService} from "../../services/user.service";
import {nullToEmptyStr} from "../../helpers/util";
import {alertActions} from "../../actions/alert.actions";
import {orderService} from "../../services/order.service";


const {Column, HeaderCell, Cell, Pagination} = Table;


class OrdersList extends React.Component {
    paymentMethods = [
        {
            value: 'WIRE_TRANSFER',
            label: 'Безналичная'
        }, {
            value: 'BANK_CARD',
            label: 'Банковская карта'
        }
    ];
    orderStatuses = [
        {
            value: 'NEW',
            label: 'Новый'
        }, {
            value: 'PENDING',
            label: 'Ждет оплаты'
        }, {
            value: 'CONFIRMED',
            label: 'Оплачено'
        }, {
            value: 'CANCELED',
            label: 'Отменен'
        }
    ];

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT OrdersList");
        const {displayLength, page, sortColumn, sortType, formValue} = this.state;

        this.jivo();

        userService.getCompanyInfo().then(companyInfo => {
            let isCompanyInfoFilled = Boolean(!!companyInfo.data.typeComp && !!companyInfo.data.legalNameComp && !!companyInfo.data.inn &&
                !!companyInfo.data.kpp && !!companyInfo.data.ogrn &&
                !!companyInfo.data.bik && !!companyInfo.data.paymentAccount &&
                !!companyInfo.data.correspondentAccount && !!companyInfo.data.bankName &&
                !!companyInfo.data.legalAddress && !!companyInfo.data.actualAddress &&
                !!companyInfo.data.nameCeo && !!companyInfo.data.namePost &&
                !!companyInfo.data.postAddress)
            this.setState({
                namePost: nullToEmptyStr(companyInfo.data.namePost),
                postAddress: nullToEmptyStr(companyInfo.data.postAddress),
                isCompanyInfoFilled: isCompanyInfoFilled
            });
        });

        this.props.dispatch(orderActions.getOrders(page, displayLength, sortColumn, sortType));
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT OrdersList");
        super(props);

        this.state = {
            displayLength: 10,
            loading: false,
            page: 1,
            sortColumn: 'created',
            sortType: 'desc',
            formValue: {
                id: '',
                statuses: []
            },
            showFilter: false,
            showDocumentsRequestModal: false,
            modalOrderId: '',
            orderStatuses: [],
            paymentMethods: [],
            namePost: '',
            postAddress: '',
            isCompanyInfoFilled: false
        };
    }

    jivo() {
        const script = document.createElement("script");

        script.src = "https://code.jivo.ru/widget/HAeYKh2mO7";
        script.async = true;

        document.body.appendChild(script);
    }

    handleChangePage = (page) => {
        const {sortColumn, sortType, displayLength, formValue, orderStatuses, paymentMethods} = this.state;

        this.setState({
            page: page
        });
        this.props.dispatch(orderActions.getOrders(page, displayLength, sortColumn, sortType));
    };

    handleChangeLength = (displayLength) => {
        const {page, sortColumn, sortType, formValue} = this.state;

        this.setState({
            page: page,
            displayLength: displayLength
        });
        this.props.dispatch(orderActions.getOrders(page, displayLength, sortColumn, sortType));
    };

    handleSortColumn = (sortColumn, sortType) => {
        const {page, displayLength} = this.state;

        this.setState({
            sortColumn,
            sortType
        });
        //get last elem splitted by dot cause column name is order.id etc
        this.props.dispatch(orderActions.getOrders(page, displayLength, sortColumn.split('.').pop(), sortType));
    };

    toggleFilter = () => {
        this.setState({
            showFilter: !this.state.showFilter
        })
    };

    filterAction = () => {
        const {displayLength, page, sortColumn, sortType, formValue} = this.state;
        if (!this.orderFilterForm.check()) {
            return;
        }
        this.props.dispatch(orderActions.getOrders(page, displayLength, sortColumn, sortType));
    };

    clearFilterInputs = () => {
        const {displayLength, page, sortColumn, sortType} = this.state;
        let filterFormValue = {
            id: '',

        };
        this.setState({formValue: filterFormValue});
        this.props.dispatch(orderActions.getOrders(page, displayLength, sortColumn, sortType));
    };

    render() {
        loyaltyLog("RENDER COMPONENT ordersList");

        const {ordersList} = this.props;
        const {displayLength, page, sortColumn, sortType, formValue, formError, isCompanyInfoFilled,
            showDocumentsRequestModal, modalOrderId, orderStatuses, paymentMethods} = this.state;
        const hasEditPermit = true;

        return (
            <div>
                <h3 className={"ll-table-header"}>{language.ORDERS_LIST}</h3>

                <Table autoHeight data={ordersList.items} loading={ordersList.loading}
                       headerHeight={styleConstants.TABLE_HEADER_ROW_HEIGHT}
                       rowHeight={styleConstants.TABLE_BODY_ROW_HEIGHT}
                       sortColumn={sortColumn}
                       sortType={sortType}
                       onSortColumn={this.handleSortColumn}
                       renderEmpty={() => <div className="rs-table-body-info">{language.NO_ORDERS}</div>}>>

                    <Column align="center" width={120} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.EXTERNAL_ORDER_ID}</HeaderCell>
                        <Cell dataKey="id"/>
                    </Column>
                    <Column align="center" width={170} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.EMAIL}</HeaderCell>
                        <Cell dataKey="email"/>
                    </Column>
                    <Column align="center" width={170} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.COMP}</HeaderCell>
                        <Cell dataKey="legalNameComp"/>
                    </Column>
                    <Column align="center" width={180} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.PAYMENT_METHOD}</HeaderCell>
                        <Cell dataKey="paymentMethod">{rowData => {
                            return this.paymentMethods.filter(v => v.value === rowData['paymentMethod'])[0].label
                        }}</Cell>
                    </Column>
                    <Column align="center" width={100} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.AMOUNT}</HeaderCell>
                        <Cell dataKey="amount"/>
                    </Column>
                    <Column align="center" width={120} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.ORDER_DATE}</HeaderCell>
                        <OrderDateCell dataKey="orderDate"/>
                    </Column>
                    <Column align="center" width={120} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.PAYMENT_DATE}</HeaderCell>
                        <OrderDateCell dataKey="paymentDate"/>
                    </Column>
                    <Column align="center" width={170} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.ORDER_STATUS}</HeaderCell>
                        <Cell dataKey="orderStatus">{rowData => {
                            return this.orderStatuses.filter(v => v.value === rowData['orderStatus'])[0].label
                        }}</Cell>
                    </Column>
                    <Column align="center" width={120} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.DOCUMENTS_REQUEST}</HeaderCell>
                        <Cell dataKey="documentsRequestedDate">
                            {rowData => {
                                if (rowData.documentsRequestedDate) {
                                    return new Date(rowData.documentsRequestedDate).toLocaleString();
                                } else {
                                    return (<Whisper placement="top" trigger="hover"
                                                     speaker={<Tooltip>{language.REQUEST_DOCUMENTS}</Tooltip>}>
                                        <IconButton
                                            appearance="subtle"
                                            onClick={
                                                () => {
                                                    this.setState({
                                                        showDocumentsRequestModal: true,
                                                        modalOrderId: rowData.id
                                                    });
                                                }}
                                            icon={<Icon icon={mailButton}/>}
                                        />
                                    </Whisper>)
                                }
                            }}
                        </Cell>
                    </Column>
                    <Column align="center" width={135} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.DOCUMENTS_RESPONSE}</HeaderCell>
                        <OrderDocResponseCell dataKey="documentsRespondedDate"
                                              documentsRequestedDate={'documentsRequestedDate'}/>
                    </Column>
                    <Column width={165} resizable align="center">
                        <HeaderCell>{language.BILL}</HeaderCell>
                        <Cell style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
                            {(rowData) => {
                                return <Whisper placement="top" trigger="hover"
                                                speaker={<Tooltip>{language.GET_ORDER}</Tooltip>}>
                                    <IconButton
                                        appearance="subtle"
                                        onClick={() => {
                                            if (!isCompanyInfoFilled) {
                                                this.props.dispatch(alertActions.error(language.COMPANY_INFO_NOT_FILLED));
                                            } else {
                                                orderService.downloadInvoice(rowData['id']).then((blob) => {
                                                    const objectURL = window.URL.createObjectURL(blob)
                                                    const link = document.createElement('a');
                                                    link.href = objectURL;
                                                    link.download = 'Invoice-' + rowData['id'] + '.docx';
                                                    document.body.appendChild(link);
                                                    link.click();
                                                    document.body.removeChild(link);
                                                })
                                            }
                                        }}
                                        icon={<Icon icon={downloadButton}/>}
                                    />
                                </Whisper>
                            }}
                        </Cell>
                    </Column>
                </Table>


                <Pagination
                    lengthMenu={[
                        {
                            value: 10,
                            label: 10
                        },
                        {
                            value: 20,
                            label: 20
                        }
                    ]}
                    activePage={page}
                    displayLength={displayLength}
                    total={ordersList.totalElements}
                    onChangePage={this.handleChangePage}
                    onChangeLength={this.handleChangeLength}
                />
                <DocumentsRequestModal showDocumentsRequestModal={showDocumentsRequestModal} orderId={modalOrderId}
                                       namePost={this.state.namePost} postAddress={this.state.postAddress}
                                       closeModal={() => {
                                           this.setState({showDocumentsRequestModal: false})
                                       }}
                                       dispatch={this.props.dispatch}
                                       pagination={{displayLength, page, sortColumn, sortType}}/>
            </div>
        );
    }
}







class DocumentsRequestModal extends Component {

    constructor(props) {
        super(props);
        this.state = {
            show: false,
            modalFormValue: {
                namePost: '',
                postAddress: ''
            },
            modalFormError: {},
            orderId: '',
            pagination: ''
        }
    }

    UNSAFE_componentWillReceiveProps(props) {
        this.setState({
            show: props.showDocumentsRequestModal,
            orderId: props.orderId,
            pagination: props.pagination,
            closeModal: props.closeModal,
            modalFormValue: {
                namePost: props.namePost || '',
                postAddress: props.postAddress || ''
            }
        })
    }

    render() {
        loyaltyLog("RENDER DOC REQUEST MODAL");

        const { modalFormValue, modalFormError, show, closeModal } = this.state;

        return (<Modal show={this.state.show} onHide={() => this.setState({show: false})}>
            <Modal.Header>
                <Modal.Title>{language.DOCUMENTS_REQUEST_TITLE}</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <p>{language.DOCUMENTS_REQUEST_BODY}</p>
                <Form style={{padding: 15}}
                      formValue={modalFormValue}
                      formError={modalFormError}
                      onChange={modalFormValue => {
                          this.setState({modalFormValue : modalFormValue});
                      }}
                      onCheck={modalFormError => this.setState({modalFormError})}
                      ref={ref => (this.form = ref)}
                      model={Schema.Model({
                          namePost: StringType().isRequired(language.REQUIRED_INPUT),
                          postAddress: StringType().isRequired(language.REQUIRED_INPUT)
                      })}
                      checkTrigger={'blur'}>

                    <FormGroup>
                        <ControlLabel>{language.NAME_POST}</ControlLabel>
                        <FormControl name="namePost" style={{width: 500}}/>
                    </FormGroup>
                    <FormGroup>
                        <ControlLabel>{language.POST_ADDRESS}</ControlLabel>
                        <FormControl name="postAddress" style={{width: 500}}/>
                    </FormGroup>

                </Form>
            </Modal.Body>
            <Modal.Footer>
                <Button onClick={() => {
                    if (!this.form.check()) {
                        return
                    }
                    this.props.dispatch(orderActions.requestDocuments(this.state.orderId, modalFormValue.namePost, modalFormValue.postAddress));
                    this.props.dispatch(orderActions.getOrders(this.state.pagination.page, this.state.pagination.displayLength, this.state.pagination.sortColumn, this.state.pagination.sortType));
                    this.setState({show: false, modalFormValue: { namePost: '', postAddress:''}});
                    closeModal();
                }} appearance="primary">
                    {language.REQUEST_DOCUMENTS}
                </Button>
                <Button onClick={() => this.setState({show: false, modalFormValue: { namePost: '', postAddress:''}})} appearance="subtle">
                    {language.CANCEL}
                </Button>
            </Modal.Footer>
        </Modal>)
    }
}

const {NumberType} = Schema.Types;
const orderFilterModel = Schema.Model({});

const OrderDateCell = ({rowData, dataKey, ...props}) => {
    return (
        <Cell {...props} className="link-group">
            {rowData[dataKey] ? new Date(rowData[dataKey]).toLocaleString() : ''}
        </Cell>
    );
};

const OrderDocRequestCell = ({rowData, dataKey, onClick, ...props}) => {
    if (rowData[dataKey]) {
        return new Date(rowData[dataKey]).toLocaleString();
    } else {
        return (
            <Whisper placement="top" trigger="hover" speaker={<Tooltip>{language.REQUEST_DOCUMENTS}</Tooltip>}>
                <IconButton
                    appearance="subtle"
                    onClick={onClick()}
                    icon={<Icon icon={mailButton}/>}
                />
            </Whisper>
        );
    }
};

const OrderDocResponseCell = ({rowData, documentsRequestedDate, dataKey, ...props}) => {
    function handleAction() {
        alert("Скоро сделаем скачивание документов");
    }

    if (rowData[dataKey]) {
        return (
            <Cell {...props} style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
                <Whisper placement="top" trigger="hover" speaker={<Tooltip>{language.DOWNLOAD_DOCUMENTS}</Tooltip>}>
                    <IconButton
                        appearance="subtle"
                        onClick={handleAction}
                        icon={<Icon icon={downloadButton}/>}
                    />
                </Whisper>
            </Cell>
        );
    } else {
        return (
            <Cell {...props} className="link-group">
                {rowData[documentsRequestedDate] ? language.IN_PROGRESS : ""}
            </Cell>
        );
    }
};

function mapStateToProps(state) {

    const {ordersList} = state;
    return {
        ordersList
    };
}

const connectedOrdersList = connect(mapStateToProps)(OrdersList);
export {connectedOrdersList as OrdersList};