import React from "react";
import {connect} from "react-redux";
import {
    ButtonToolbar, Checkbox,
    ControlLabel,
    Form,
    FormControl,
    FormGroup, Icon, IconButton, Input, Schema, SelectPicker
} from "rsuite";

import {pathConstants} from "../../constants/path.constants";
import {nullToEmptyArray, nullToEmptyStr, nullToFalse} from "../../helpers/util";
import {language} from "../../content/language";
import {questionService} from "../../services/question.service";
import {questionActions} from "../../actions/question.actions";
import {CreateObjectButton} from "../buttons/CreateObjectButton";
import {alertActions} from "../../actions/alert.actions";
import {focusError} from "../../helpers/form.helper";
import {BackTextLink} from "../buttons/BackTextLink";
import {history} from "../../helpers/history";
import {hasAuthority} from "../../helpers/auth";
import deleteButton from "../../images/trash_icon.svg";
import addAnswer from "../../images/add_answer_options.svg";
import {authoritiesConstants} from "../../constants/authorities.constants";
import saveButton from "../../images/save_button.svg";
import {SaveButton} from "../buttons/SaveButton";
import loyaltyLog from "../../helpers/loyaltyLog";

const editorConfiguration = {
    toolbar: [ 'heading', 'codeBlock', 'bold', 'italic', 'underline','bulletedList', 'numberedList',
        'fontSize', 'fontColor', 'undo', 'redo', 'outdent', 'indent']
};

class QuestionDetails extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT QUESTION DETAILS DETAILS");

        let topicId = this.props.match.params.id;

        //TODO create (level and type) Загрузка справочника статусов
        this.setState({
            questionLevels: [{
                    label: 'Junior',
                    value: 'Junior'
                },
                {
                    label: 'Middle',
                    value: 'Middle'
                },
                {
                    label: 'Senior',
                    value: 'Senior'
                }],
            questionTypes: [{
                    label: 'Один выбор',
                    value: 'Single'
                },
                {
                    label: 'Множественный выбор',
                    value: 'Multiple'
                }]
        });

        //Если создание выставляем пустую форму и даем возможность редактировать поле по значению creating
        if (this.props.match.path === pathConstants.QUESTION_CREATE) {
            this.setState({
                formValue: {
                    id: '',
                    description: '',
                    increasedComplexity: false,
                    topicId: topicId,
                    type: '',
                    level: '',
                    answers: [{
                            description: '',
                            correct: false
                        },
                        {
                            description: '',
                            correct: false
                        }
                    ],
                    maxAnswers: 6,
                    tags: []
                },
                creating: true
            });
            //Если редактирование загружаем данные вопроса.
        } else {
            questionService.getQuestionById(this.props.match.params.questionId).then(question => {
                    this.setState({
                        formValue: {
                            id: nullToEmptyStr(question.data.id),
                            description: nullToEmptyStr(question.data.description),
                            increasedComplexity:  nullToFalse(question.data.increasedComplexity),
                            topicId: nullToEmptyStr(question.data.topicId),
                            type: nullToEmptyStr(question.data.type),
                            level: nullToEmptyStr(question.data.level),
                            answers: nullToEmptyArray(question.data.answers),
                            maxAnswers: 6,
                            tags: nullToEmptyArray(question.data.tags)
                        },
                        creating: false
                    });
                }
            );
        }
    };

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT QUESTION DETAILS");
        super(props);
        this.state = {
            formValue: {
                id: '',
                description: '',
                increasedComplexity: '',
                topicId: this.props.match.params.id,
                type: '',
                level: '',
                answers: [],
                maxAnswers: 6,
                tags: []
            },
            formError: {},
            questionLevels: [],
            questionTypes: [],
            creating: false
        }
    };

    addAnswer = () => {
        let type = this.checkQuestionType(null);
        if (this.state.formValue.answers.length >= this.state.formValue.maxAnswers) {
            return;
        }
        const emptyAnswer = {
            description: '',
            correct: false
        };
        let newAnswers = this.state.formValue.answers;
        newAnswers.push(emptyAnswer);
        this.setState({
            id: this.state.formValue.id,
            description: this.state.formValue.description,
            increasedComplexity: this.state.formValue.increasedComplexity,
            topicId: this.state.formValue.topicId,
            type: type,
            level: this.state.formValue.level,
            answers: newAnswers,
            maxAnswers: 6,
            tags: this.state.formValue.tags
        });
    };

    deleteAnswer = (index) => {
        let newAnswers = this.state.formValue.answers;
        newAnswers.splice(index, 1);
        let type = this.checkQuestionType(null);
        this.setState({
            formValue: {
                id: this.state.formValue.id,
                description: this.state.formValue.description,
                increasedComplexity: this.state.formValue.increasedComplexity,
                topicId: this.state.formValue.topicId,
                type: type,
                level: this.state.formValue.level,
                answers: newAnswers,
                maxAnswers: 6,
                tags: this.state.formValue.tags
            }
        });
    };

    updAnswerDescription = (index, value) => {
        let newAnswers = this.state.formValue.answers;
        newAnswers[index].description = value;
        this.setState({
            answers: newAnswers
        });
    };

    updAnswerCorrect = (index, checked) => {
        let type = this.checkQuestionType(checked);
        let newAnswers = this.state.formValue.answers;
        newAnswers[index].correct = checked;
        this.setState({
            formValue: {
                id: this.state.formValue.id,
                description: this.state.formValue.description,
                increasedComplexity: this.state.formValue.increasedComplexity,
                topicId: this.state.formValue.topicId,
                type: type,
                level: this.state.formValue.level,
                answers: newAnswers,
                maxAnswers: 6,
                tags: this.state.formValue.tags
            }
        });
    };

    checkQuestionType = (checked) => {
        let checkedAns = 0;
        this.state.formValue.answers.forEach(ans => {
            if (ans.correct) {
                checkedAns++;
            }
        });
        if (null != checked) {
            if ((checkedAns == 0 && checked) || (checkedAns == 2 && !checked)) {
                return 'Single';
            } else if ((checkedAns > 0 && checked) || (checkedAns > 1 && !checked)) {
                return 'Multiple';
            }
        } else {
            if (checkedAns == 1) {
                return 'Single';
            } else if (checkedAns > 1) {
                return 'Multiple';
            }
        }
        return '';
    };

    render() {

        loyaltyLog("RENDER COMPONENT QUESTION DETAILS");

        const { formValue, creating, formError, questionLevels, questionTypes } = this.state;

        const hasEditPermit = hasAuthority(authoritiesConstants.ROLE_EDITOR);

        return (
            <Form className={"ll-form-page"} layout="horizontal" readOnly={!hasEditPermit}
                  onChange={formValue => this.setState({ formValue }) } formValue={formValue}
                  ref={ref => (this.form = ref)}
                  onCheck={formError => this.setState({ formError })} formError={formError} model={questionModel}
                  onError={errors => focusError(errors) }>

                <FormGroup className={"ll-form-header"}>
                    <h3>{creating ? language.CREATE_QUESTION : language.EDIT_QUESTION}</h3>
                    <BackTextLink action={() => history.push(`/sections/topics/${this.props.match.params.id}/questions`)} text={language.BACK_TO_QUESTION_LIST}/>
                </FormGroup>

                <FormGroup style={{height: 0, opacity: 0}}>
                    <ControlLabel style={{opacity: 0, height: 0, margin: 0}}/>
                    <FormControl name="description" style={{opacity: 0, height:0 }}/>
                </FormGroup>
                <FormGroup>
                    <ControlLabel>{language.DESCRIPTION}</ControlLabel>
                </FormGroup>


                <FormGroup>
                    <Checkbox name="increasedComplexity" style={{width: 505}} checked={formValue.increasedComplexity} onChange={(v, checked) => {
                        formValue.increasedComplexity = checked;
                        this.setState({ formValue });
                    }} >{language.INCREASED_COMPLEXITY}</Checkbox>
                </FormGroup>

                <FormGroup>
                    <ControlLabel>{language.QUESTION_TYPE}</ControlLabel>
                    <FormControl accepter={SelectPicker} placeholder={language.CHOOSE_QUESTION_TYPE} name="type" disabled={true}
                                 data={questionTypes} searchable={false} style={{width: 505}} cleanable={false}/>
                </FormGroup>

                <FormGroup name="answers">
                    <h4 style={{marginBottom: 20}}>{language.ANSWER_OPTIONS}</h4>
                    <AnswerInputs answers={this.state.formValue.answers} delAction={this.deleteAnswer}
                            updDescription={this.updAnswerDescription} updCorrect={this.updAnswerCorrect}/>
                </FormGroup>

                <FormGroup>
                    <ControlLabel></ControlLabel>
                    <CreateObjectButton hasEditPermit={this.state.formValue.answers.length < this.state.formValue.maxAnswers ? hasEditPermit : false}
                                        className={"create-answer-button"}
                                        onClickAction={this.addAnswer} icon={<Icon icon={addAnswer}/>}/>
                </FormGroup>

                <FormGroup style={{marginTop: 60}}>
                    <ButtonToolbar>
                        <UpdateQuestionButton hasEditPermit={hasEditPermit} questionForm={formValue} parent={this}
                                                  topicId={this.props.match.params.id} dispatch={this.props.dispatch} isNew={creating}/>
                    </ButtonToolbar>
                </FormGroup>
            </Form>
        )
    };
}

const { StringType } = Schema.Types;
const questionModel = Schema.Model({
    description: StringType().isRequired(language.REQUIRED_INPUT)
});

const AnswerInputs = ({ answers, delAction, updDescription, updCorrect, ...props }) => (
    <>
        {
            answers.map((answer, index) => (
                <div key={index} style={{display: 'flex'}}>
                    <ControlLabel></ControlLabel>
                    <Input name={'description' + index} key={index} rows={1} onChange={(value) => {updDescription(index, value);}}
                           value={answers[index].description} componentClass="textarea"
                           style={{width: 505, resize: 'auto', marginBottom: 20}}/>
                    <Checkbox name={'correct' + index} checked={answers[index].correct}
                                 value={answers[index].correct} onChange={(value, checked) => {updCorrect(index, checked);}}>
                        {language.CORRECT}
                    </Checkbox>
                    <DeleteAnswerOption index={index} delAction={delAction} answers={answers}/>
                </div>
            ))
        }
    </>
);

const DeleteAnswerOption = ({index, answers, delAction}) => {
    if (answers.length <= 2) {
        return null;
    }
    function action() {
        delAction(index);
    }
    return (
        <IconButton
            appearance="subtle"
            onClick={action}
            icon={<Icon icon={deleteButton} />}
            style={{marginLeft: 20}}
        />
    );
};

const UpdateQuestionButton = ({questionForm, hasEditPermit, parent, isNew}) => {

    if (hasEditPermit === false) {
        return null;
    }

    function updateQuestionDetails() {
        if (!parent.form.check()) {
            return;
        }
        if (customAnswersCheck(questionForm)) {
            parent.props.dispatch(alertActions.error(language.FILL_ALL_ANSWER_OPTIONS));
            return;
        }
        if (isNew) {
            parent.props.dispatch(questionActions.createQuestion(questionForm));
        } else {
            parent.props.dispatch(questionActions.updateQuestion(questionForm));
        }
    }
    function customAnswersCheck(questionForm) {
        let hasErrors = false;
        let atLeastOneChecked = false;
        questionForm.answers.forEach(answer => {
            if (answer.correct) {
                atLeastOneChecked = true;
            }
            if (answer.description == null || answer.description === '') {
                hasErrors = true;
            }
        });
        if (!atLeastOneChecked) hasErrors = true;
        return hasErrors;
    }
    return (
        <SaveButton hasEditPermit={hasEditPermit} icon={<Icon icon={saveButton}/>}
            onClickAction={updateQuestionDetails}/>
    );
};

function mapStateToProps(state) {
    const { questionDetails } = state;
    return {
        questionDetails
    };
}

const connectedQuestionDetails = connect(mapStateToProps)(QuestionDetails);
export { connectedQuestionDetails as QuestionDetails };
