import React from "react";
import {connect} from "react-redux";
import {
    ButtonToolbar, CheckPicker, Col,
    ControlLabel,
    Form,
    FormControl,
    Checkbox,
    FormGroup, Icon, Row, SelectPicker, Tooltip, Whisper
} from "rsuite";
import {testActions} from "../../actions/test.actions";
import {testService} from "../../services/test.service";
import {language} from "../../content/language";
import {Schema} from 'rsuite';
import {nullToEmptyArray, nullToEmptyStr, nullToFalse, nullToZero} from "../../helpers/util";
import {pathConstants} from "../../constants/path.constants";
import {topicService} from "../../services/topic.service";
import {BackTextLink} from "../buttons/BackTextLink";
import {history} from "../../helpers/history";
import saveButton from "../../images/save_button.svg";
import {SaveButton} from "../buttons/SaveButton";
import {testConstants} from "../../constants/test.constants";
import {focusError} from "../../helpers/form.helper";
import loyaltyLog from "../../helpers/loyaltyLog";
import {alertActions} from "../../actions/alert.actions";
import {responseMessages} from "../../content/responseMessages";


class TestDetails extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT TEST DETAILS");

        let testId = this.props.match.params.id;

        const size = 10000;
        topicService.getTopics(0, size, '', '').then(sections => {
            let tagPickerTopics = [];
            sections.data.forEach(section => {
                section.topics.forEach(topic => {
                    tagPickerTopics.push({
                        section: section.name,
                        label: topic.name,
                        value: topic.id
                    })
                })
            });
            this.setState({topics: tagPickerTopics});
        });

        //Если создание выставляем пустую форму и даем возможность редактировать поле по значению creating
        if (this.props.match.path === pathConstants.TEST_CREATE) {
            this.setState({
                formValue: {
                    id: '',
                    name: '',
                    duration: '',
                    numberOfQuestions: '',
                    description: '',
                    level: '',
                    increasedComplexity: false,
                    topics: []
                },
                easyQuestionsCount: 0,
                allQuestionsCount: 0,
                creating: true
            });
            //Если редактирование загружаем данные вопроса.
        } else {
            testService.getTestById(testId).then(response => {
                    this.setState({
                        formValue: {
                            id: nullToEmptyStr(response.data.id),
                            name: nullToEmptyStr(response.data.name),
                            duration: Math.floor(nullToZero(response.data.duration) / 60),
                            numberOfQuestions: nullToEmptyStr(response.data.questionNumber),
                            description: nullToEmptyStr(response.data.description),
                            level: nullToEmptyStr(response.data.level),
                            increasedComplexity: nullToFalse(response.data.increasedComplexity),
                            topics: nullToEmptyArray(response.data.topicIds)
                        },
                        easyQuestionsCount: 0,
                        allQuestionsCount: 0,
                        creating: false
                    });
                }
            );
        }
    };

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT TEST DETAILS");
        super(props);

        this.state = {
            formValue: {
                name: '',
                duration: '',
                numberOfQuestions: '',
                description: '',
                level: '',
                increasedComplexity: false,
                topics: []
            },
            formError: {},
            levels: [
                {
                    label: 'Junior',
                    value: 'Junior'
                }, {
                    label: 'Middle',
                    value: 'Middle'
                }, {
                    label: 'Senior',
                    value: 'Senior'
                }
            ],
            durations: [
                {
                    label: '30',
                    value: '30'
                }, {
                    label: '45',
                    value: '45'
                }, {
                    label: '60',
                    value: '60'
                }
            ],
            topics: [],
            creating: false,
            easyQuestionsCount: 0,
            allQuestionsCount: 0,
        }
    };

    render() {
        loyaltyLog("RENDER COMPONENT TEST DETAILS");

        const {formValue, topics, levels, formError, creating, durations} = this.state;
        const hasEditPermit = true;

        return (
            <Form className={"ll-form-page"} readOnly={!hasEditPermit}
                  onChange={formValue => {
                      this.setState({formValue});
                  }} formValue={formValue}
                  ref={ref => (this.form = ref)}
                  onCheck={formError => this.setState({formError})} formError={formError} model={testModel}
                  onError={errors => focusError(errors)}>

                <FormGroup className={"ll-form-header"}>
                    <h3>{creating ? language.CREATE_TEST : language.EDIT_TEST}</h3>
                    <BackTextLink action={() => history.push(pathConstants.TESTS_LIST)}
                                  text={language.BACK_TO_TEST_LIST}/>
                </FormGroup>

                <Row>
                    <Col xs={24} sm={24} md={15}>
                        <FormGroup>
                            <ControlLabel>{language.TEST_TITLE_FOR_CANDIDATE}</ControlLabel>
                            <FormControl name="name" style={{width: 400}}/>
                        </FormGroup>

                        <FormGroup>
                            <ControlLabel>{language.TEST_DURATION}</ControlLabel>
                            <Whisper trigger="focus" placement={"left"}
                                     speaker={<Tooltip>{language.RECOMMEND_Q_TIME_DEFAULT}</Tooltip>}>
                                <FormControl accepter={SelectPicker} placeholder={language.CHOOSE_TEST_DURATION}
                                             name="duration" data={durations} searchable={false}
                                             style={{width: 400}}
                                             onChange={(duration) => {
                                                 formValue.duration = duration;
                                                 formValue.numberOfQuestions = Math.ceil(formValue.duration / testConstants.RECOMMEND_QUESTION_TIME);
                                                 this.setState({formValue});
                                             }}
                                             disabled={!creating}
                                />
                            </Whisper>
                        </FormGroup>

                        <FormGroup>
                            <ControlLabel>{language.QUESTIONS_NUMBER}</ControlLabel>
                            <FormControl style={{width: 400}} name="numberOfQuestions" disabled={true}/>
                        </FormGroup>

                        <FormGroup>
                            <Checkbox name="increasedComplexity" checked={formValue.increasedComplexity}
                                      onChange={(v, checked) => {
                                          formValue.increasedComplexity = checked;
                                          this.setState({formValue});
                                          this.countQuestionsByTopicIds(this.state.formValue.topics);
                                      }}>{language.INCREASED_COMPLEXITY}</Checkbox>
                        </FormGroup>

                    </Col>
                    <Col xs={24} sm={24} md={9}>
                        <FormGroup>
                            <ControlLabel>{language.TOPICS}</ControlLabel>
                            <FormControl accepter={CheckPicker} placeholder={language.CHOOSE_TEST_TOPICS} data={topics}
                                         groupBy={"section"} searchable={true} style={{width: 400}} name="topics"
                                         disabled={!creating}
                                         onChange={(value) => {
                                             if (value.length > 0) {
                                                 testService.countQuestionsByTopics(value).then(
                                                     response => {
                                                         this.setState({
                                                             easyQuestionsCount: response.easyQuestionsCount,
                                                             allQuestionsCount: response.allQuestionsCount
                                                         })
                                                         this.countQuestionsByTopicIds(value);
                                                     })
                                             }
                                         }}
                            />
                        </FormGroup>
                    </Col>
                </Row>

                <FormGroup>
                    <ButtonToolbar>
                        <UpdateTestDetailsButton hasEditPermit={hasEditPermit} testForm={formValue}
                                                 dispatch={this.props.dispatch} parent={this} isNew={creating}/>
                    </ButtonToolbar>
                </FormGroup>
            </Form>
        )
    };

    countQuestionsByTopicIds(topicIds) {
        if (topicIds.length <= 0) {
            this.setState({durations: []})
        } else {
            let durations = [], max;
            if (this.state.formValue.increasedComplexity) {
                max = Math.min(this.state.allQuestionsCount * testConstants.RECOMMEND_QUESTION_TIME, 120);
            } else {
                max = Math.min(this.state.easyQuestionsCount * testConstants.RECOMMEND_QUESTION_TIME, 120);
            }
            for (let i = 30; i<=max; i+=15){
                durations.push({
                    label: i.toString(),
                    value: i.toString()
                })
            }
            this.setState({
                durations: durations
            })
        }
    }
}

const {StringType, NumberType, ObjectType} = Schema.Types;
const testModel = Schema.Model({
    name: StringType().isRequired(language.REQUIRED_INPUT),
    duration: NumberType(language.INSERT_NUMBER).isRequired(language.REQUIRED_INPUT).min(1, language.MIN_DURATION),
    numberOfQuestions: NumberType(language.INSERT_NUMBER)
        .isRequired(language.REQUIRED_INPUT).min(1, language.MIN_QUESTION_NUMBER),
    topics: ObjectType().isRequired(language.REQUIRED_INPUT)
});

const UpdateTestDetailsButton = (props) => {
    if (props.hasEditPermit === false) {
        return null;
    }

    function updateTestDetails() {

        if (!props.parent.form.check()) {
            return;
        }
        if (props.isNew) {
            props.dispatch(testActions.createTest(props.testForm));
        } else {
            props.dispatch(testActions.updateTest(props.testForm));
        }
    }

    return (
        <SaveButton hasEditPermit={props.hasEditPermit} icon={<Icon icon={saveButton}/>}
                    onClickAction={updateTestDetails}/>
    );
};

function mapStateToProps(state) {
    const {testDetails} = state;
    return {
        testDetails
    };
}

const connectedTestDetails = connect(mapStateToProps)(TestDetails);
export {connectedTestDetails as TestDetails};