import React from "react";
import {connect} from "react-redux";
import {
    Col,
    Icon,
    IconButton, Row, Schema,
    Table
} from "rsuite";
import {language} from "../../content/language";
import {styleConstants} from "../../constants/style.constants";
import {CreateObjectButton} from "../buttons/CreateObjectButton";
import createSectionButton from "../../images/create_section_button.svg";
import createTopicButton from "../../images/create_topic_button.svg";
import {Link} from "react-router-dom";
import loyaltyLog from "../../helpers/loyaltyLog";
import {topicActions} from "../../actions/topic.action";

const { Column, HeaderCell, Cell } = Table;


class TopicsList extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT TOPICS LIST");
        const {displayLength, page, sortColumn, sortType, formValue} = this.state;

        this.props.dispatch(topicActions.getTopics(page, displayLength, sortColumn, sortType));
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT TOPICS LIST");
        super(props);

        this.state = {
            displayLength: 10000,
            loading: false,
            page: 1,
            sortColumn: 'id',
            sortType: 'asc',
            formValue: {
                id: '',
                statuses: []
            },
            topicsStatuses: [],
            showFilter: false
        };
    }

    handleChangePage = (page) => {
        const { sortColumn, sortType, displayLength, formValue } = this.state;

        this.setState({
            page: page
        });
        this.props.dispatch(topicActions.getTopics(page, displayLength, sortColumn, sortType));
    };

    handleChangeLength = (displayLength) => {
        const { page, sortColumn, sortType, formValue } = this.state;

        this.setState({
            page: page,
            displayLength: displayLength
        });
        this.props.dispatch(topicActions.getTopics(page, displayLength, sortColumn, sortType));
    };

    handleSortColumn = (sortColumn, sortType) => {
        const { page, displayLength,formValue } = this.state;

        this.setState({
            sortColumn,
            sortType
        });
        this.props.dispatch(topicActions.getTopics(page, displayLength, sortColumn, sortType));
    };

    toggleFilter = () => {
        this.setState({
            showFilter: !this.state.showFilter
        })
    };

    filterAction = () => {
        const { displayLength, page, sortColumn, sortType, formValue } = this.state;
        if (!this.topicFilterForm.check()) {
            return;
        }
        this.props.dispatch(topicActions.getTopics(page, displayLength, sortColumn, sortType));
    };

    clearFilterInputs = () => {
        const { displayLength, page, sortColumn, sortType } = this.state;
        let filterFormValue = {
            id: '',
            name: '',
            address: '',
            serialNumber: '',
            ratingFrom: '',
            ratingTo: '',
            statuses: []
        };
        this.setState({ formValue: filterFormValue });
        this.props.dispatch(topicActions.getTopics(page, displayLength, sortColumn, sortType));
    };

    render() {

        loyaltyLog("RENDER COMPONENT TOPIC LIST");

        const { topicsList } = this.props;

        const { displayLength, page, sortColumn, sortType, formValue, topicsStatuses, formError } = this.state;

        return (
            <div>
                <Row className={"ll-table-header"}>
                    <Col xs={12}>
                        <h3>{language.SECTIONS_LIST}</h3>
                    </Col>
                    <Col xs={12}>
                        <CreateObjectButton hasEditPermit={true} icon={<Icon icon={createSectionButton}/>}
                                            className={"create-topic-button"}
                                            onClickAction={() => alert('Скоро будет доступно')/*history.push(pathConstants.SECTION_CREATE)*/}/>
                    </Col>
                    <Col xs={24} style={{marginTop: 18}}>
                        <CreateObjectButton hasEditPermit={true} icon={<Icon icon={createTopicButton}/>}
                                            className={"create-topic-button"}
                                            onClickAction={() => alert('Скоро будет доступно')/*history.push(pathConstants.TOPIC_CREATE)*/}/>
                    </Col>
                </Row>

                <Table autoHeight data={topicsList.items} loading={topicsList.loading}
                       headerHeight={styleConstants.TABLE_HEADER_ROW_HEIGHT}
                       rowHeight={styleConstants.TABLE_BODY_ROW_HEIGHT}
                       sortColumn={sortColumn}
                       sortType={sortType}
                       onSortColumn={this.handleSortColumn}>

                    <Column width={55} resizable sortable style={{paddingLeft: 0}}>
                        <HeaderCell>{language.ID}</HeaderCell>
                        <Cell dataKey="id" />
                    </Column>

                    <Column width={250} resizable sortable>
                        <HeaderCell>{language.TITLE}</HeaderCell>
                        <QuestionsCell dataKey={'name'} />
                    </Column>

                </Table>
                {/*<Pagination
                    lengthMenu={[
                        {
                            value: 10,
                            label: 10
                        },
                        {
                            value: 20,
                            label: 20
                        }
                    ]}
                    activePage={page}
                    displayLength={displayLength}
                    total={topicsList.totalElements}
                    onChangePage={this.handleChangePage}
                    onChangeLength={this.handleChangeLength}
                />*/}
            </div>
        );
    }
}

const { NumberType } = Schema.Types;
const topicFilterModel = Schema.Model({
    id: NumberType(language.INSERT_NUMBER)
});

const QuestionDetailsCell = ({ rowData, dataKey, ...props }) => {
    function handleAction() {
        //history.push(`/topics/${rowData[dataKey]}`);
        alert("NOT IMPL YET");
    }
    return (
        <Cell {...props} style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
            <IconButton
                appearance="subtle"
                onClick={handleAction}
                icon={<Icon icon="edit2" />}
            />
        </Cell>
    );
};

const QuestionsCell = ({ rowData, dataKey, ...props }) => {
    return (
        <Cell {...props}>
            <Link to={`/sections/topics/${rowData['id']}/questions`}>{rowData[dataKey]}</Link>
        </Cell>
    );
};

function mapStateToProps(state) {

    const { topicsList} = state;
    return {
        topicsList
    };
}

const connectedTopicsList = connect(mapStateToProps)(TopicsList);
export { connectedTopicsList as TopicsList };