import config from 'config';
import {apiConstants} from '../constants/api.constants';
import {authHeader, authHeaderPdf} from "../helpers/auth.header";
import {handleResponse} from "./response.handler";

export const orderService = {
    getOrders,
    createOrder,
    requestDocuments,
    downloadInvoice,
    downloadOffer
};


function getOrders(page, size, sort, sortType) {
    //page count from 0. passed from 1
    page = page ? page - 1  : '';
    size = size ? size : '';
    sort = sort ? sort : '';
    sortType = sortType ? sortType : '';

    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.ORDERS}?page=${page}&size=${size}&sort=${sort},${sortType}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function requestDocuments(orderId, documentsPostName, documentsPostAddress) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: JSON.stringify({
            "orderId": orderId,
            "documentsPostName": documentsPostName,
            "documentsPostAddress": documentsPostAddress
        })
    };
    return fetch(`${config.apiUrl}${apiConstants.ORDERS_REQUEST_DOCUMENTS}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function downloadInvoice(orderId) {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.DOWNLOAD_INVOICE(orderId)}`, requestOptions).then(response => response.blob());
}

function downloadOffer() {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.DOWNLOAD_OFFER}`, requestOptions).then(response => response.blob());
}

function createOrder(orderModel) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: orderModelToJSONBody(orderModel)
    };
    return fetch(`${config.apiUrl}${apiConstants.ORDERS}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}


function orderModelToJSONBody(orderModel) {
    return JSON.stringify({
        "userId": orderModel.userId,
        "rateId": orderModel.rateId,
        "paymentMethod": orderModel.paymentMethod,
    });
}
