import {authHeader} from "../helpers/auth.header";
import config from "config";
import {apiConstants} from "../constants/api.constants";
import {handle500, handleResponse} from "./response.handler";

export const questionService = {
    getQuestions,
    getQuestionById,
    createQuestion,
    updateQuestion,
    deleteQuestion
};


function getQuestions(topicId, page, size, sort, sortType) {
    //page count from 0. passed from 1
    page = page ? page - 1  : '';
    size = size ? size : '';
    sort = sort ? sort : '';
    sortType = sortType ? sortType : '';

    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TOPIC_QUESTIONS(topicId)}?page=${page}&size=${size}&sort=${sort},${sortType}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}

function getQuestionById(id) {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.QUESTION(id)}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}
/*
    QuestionModel

    "id":3,
	"description": "Что за фреймворк Spring??",
	"topicId": 1,
	"type": "MULTIPLE",
	"level": "MEDIUM",
	"answers": [
		{
			"description":"Это так себе фреймворККК",
			"correct":false
		},
		{
			"description":"Этой очень крутой фреймворк",
			"correct":true
		}],
    "tags": [
    	{
            "id": 1,
            "name": "Java 8"
        }]

 */
function createQuestion(questionModel) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: questionModelToJSONBody(questionModel)
    };
    return fetch(`${config.apiUrl}${apiConstants.QUESTIONS}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}
function deleteQuestion(id) {
    const requestOptions = {
        method: 'DELETE',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.QUESTION(id)}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}
function updateQuestion(questionModel) {
    const requestOptions = {
        method: 'PUT',
        headers: authHeader(),
        body: questionModelToJSONBody(questionModel)
    };
    return fetch(`${config.apiUrl}${apiConstants.QUESTIONS}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}

function questionModelToJSONBody(questionModel) {
    questionModel.answers.forEach(answer => {
        delete answer['id'];
    });
    return JSON.stringify({
        "id": questionModel.id,
        "description": questionModel.description,
        "increasedComplexity": questionModel.increasedComplexity,
        "topicId": questionModel.topicId,
        "type": questionModel.type,
        "level": questionModel.level,
        "answers": questionModel.answers
    });
}