import {userService} from "./user.service";
import {history} from "../helpers/history";
import {responseMessages} from "../content/responseMessages";

export function handleResponse(response, code400, code401, code404, code409, code500) {
    return response.text().then(text => {
        const data = text && JSON.parse(text);
        if (!response.ok) {
            const error = (data && data.errorMessage) || response.statusText;
            switch (response.status) {
                case 400:
                    if (code400) {
                        return code400(error);
                    }
                    break;
                case 401:
                    if (code401) {
                        return code401();
                    }
                    // auto logout if 401 response returned from api
                    userService.logout();
                    history.push("/");
                    Promise.reject(responseMessages.INCORRECT_LOGIN);
                    break;
                case 404:
                    if (code404) {
                        return code404();
                    }
                    break;
                case 409:
                    if (code409) {
                        return code409(data);
                    }
                    break;
                case 500:
                    if (code500) {
                        return code500();
                    }
                    break;
            }
        }
        if (data.status === 'fail') {
            if (data.data && data.data.error && data.data.error.message) {
                return Promise.reject(data.data.error.message);
            }
            return handle500();
        }
        return data;
    });
}

export function handle500() {
    return Promise.reject(responseMessages.SERVER_INTERNAL_ERROR);
}