import {authHeader} from "../helpers/auth.header";
import config from "config";
import {apiConstants} from "../constants/api.constants";
import {handle500, handleResponse} from "./response.handler";

export const sectionService = {
    getSections,
    getSectionById,
    createSection,
    updateSection,
    deleteSection
};


function getSections(page, size, sort, sortType) {
    //page count from 0. passed from 1
    page = page ? page - 1  : '';
    size = size ? size : '';
    sort = sort ? sort : '';
    sortType = sortType ? sortType : '';

    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TOPICS}?page=${page}&size=${size}&sort=${sort},${sortType}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}

function getSectionById(id) {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TOPIC(id)}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}

function createSection(sectionModel) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: sectionModel
    };
    return fetch(`${config.apiUrl}${apiConstants.TOPICS}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}
function deleteSection(id) {
    const requestOptions = {
        method: 'DELETE',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TOPIC(id)}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}
function updateSection(sectionModel) {
    const requestOptions = {
        method: 'PUT',
        headers: authHeader(),
        body: sectionModel
    };
    return fetch(`${config.apiUrl}${apiConstants.TOPICS}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        handle500
    ));
}