import config from 'config';
import { apiConstants } from '../constants/api.constants';
import { responseMessages } from '../content/responseMessages';
import { authHeader} from "../helpers/auth.header";
import {handleResponse} from "./response.handler";
import {getArrayUrlParams, getFromToUrlParams} from "../helpers/util";

export const testService = {
    getTests,
    getTestById,
    createTest,
    updateTest,
    deleteTest,
    createCandidateTest,
    getCandidateTests,
    getCandidateTestById,
    getCandidateQuestionsById,
    countQuestionsByTopics
};


function getTests(page, size, sort, sortType) {
    //page count from 0. passed from 1
    page = page ? page - 1  : '';
    size = size ? size : '';
    sort = sort ? sort : '';
    sortType = sortType ? sortType : '';

    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TESTS}?page=${page}&size=${size}&sort=${sort},${sortType}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function getCandidateTests(page, size, sort, sortType) {
    //page count from 0. passed from 1
    page = page ? page - 1  : '';
    size = size ? size : '';
    sort = sort ? sort : '';
    sortType = sortType ? sortType : '';

    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.CANDIDATE_TESTS}?page=${page}&size=${size}&sort=${sort},${sortType}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function getCandidateTestById(id) {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.CANDIDATE_TEST(id)}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function getCandidateQuestionsById(id) {
    const size = 10000;
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.CANDIDATE_TEST_QUESTIONS(id)}?size=${size}`, requestOptions)
        .then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function getTestById(id) {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TEST(id)}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function createTest(testModel) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: testModelToJSONBody(testModel)
    };
    return fetch(`${config.apiUrl}${apiConstants.TESTS}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function createCandidateTest(testId, candidateName) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: JSON.stringify({testId, candidateName})
    };
    return fetch(`${config.apiUrl}${apiConstants.CANDIDATE_TESTS}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function updateTest(testModel) {
    const requestOptions = {
        method: 'PUT',
        headers: authHeader(),
        body: testModelToJSONBody(testModel)
    };
    return fetch(`${config.apiUrl}${apiConstants.TESTS}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function deleteTest(id) {
    const requestOptions = {
        method: 'DELETE',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.TEST(id)}`, requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function countQuestionsByTopics(topicIds) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: JSON.stringify({
            "topicIds": topicIds
        })
    };
    return fetch(`${config.apiUrl}${apiConstants.TESTS_COUNT}`,
        requestOptions).then(response => handleResponse(response,
        null,
        null,
        null,
        null,
        null
    ));
}

function testModelToJSONBody(testModel) {
    return JSON.stringify({
        "id": testModel.id,
        "name": testModel.name,
        "description": testModel.description,
        "duration": testModel.duration * 60,
        "increasedComplexity": testModel.increasedComplexity,
        "questions": testModel.questions,
        "topicIds": testModel.topics,
        "questionNumber": testModel.numberOfQuestions
    });
}