var HtmlWebpackPlugin = require('html-webpack-plugin');

const webpack = require('webpack');
const dotenv = require('dotenv');
const path = require("path");

module.exports = env => {

    const fs = require('fs'); // to check if the file exists
    const currentPath = path.join(__dirname);
    const basePath = currentPath + '/.env';
    const envPath = basePath + '.' + env.ENVIRONMENT;

    const finalPath = fs.existsSync(envPath) ? envPath : basePath;
    const fileEnv = dotenv.config({ path: finalPath }).parsed;

    // reduce it to a nice object, the same as before
    const envKeys = Object.keys(fileEnv).reduce((prev, next) => {
        prev[`process.env.${next}`] = JSON.stringify(fileEnv[next]);
        return prev;
    }, {});

    return {
        mode: "development",
        resolve: {
            extensions: ['.js', '.jsx']
        },
        output: {
            filename: '[name].[contenthash].js',
            publicPath: process.env.PUBLIC_URL
        },
        module: {
            rules: [
                {
                    test: /\.(js|jsx)$/,
                    exclude: /node_modules/,
                    loader: 'babel-loader'
                },
                {
                    test: /\.css$/i,
                    use: ['style-loader', 'css-loader'],
                },
                {
                    test: /\.html$/,
                    use: [
                        {
                            loader: "html-loader"
                        }
                    ]
                },
                {
                    test: /\.svg$/,
                    use: [
                        {
                            loader: 'svg-sprite-loader',
                            options: {
                                symbolId: 'icon-[name]'
                            }
                        }
                    ],
                },
                {
                    test: /\.less$/,
                    use: [
                        { loader: "style-loader" },
                        { loader: "css-loader" },
                        {
                            loader: "less-loader",
                            options: {
                                lessOptions: {
                                    javascriptEnabled: true,
                                }
                            }
                        }]
                },
            ]
        },
        plugins: [new HtmlWebpackPlugin({
            template: './src/index.html',
            filename: "./index.html"
        }),
            new webpack.DefinePlugin(envKeys)
        ],
        devServer: {
            contentBase: path.join(__dirname, "dist"),
            compress: true,
            port: 4200,
            watchContentBase: true,
            progress: true,
            historyApiFallback: true
        },
        externals: {
            // global app config object
            config: JSON.stringify({
                apiUrl: process.env.API_URL
            })
        }
    }
};